/***************************************************************************
                          bibentry.cpp  -  description
                             -------------------
    begin                : Sat May 24 2003
    copyright            : (C) 2003 by Thach Nguyen
    email                : thach@dragon.thach.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <map>
#include <stdlib.h>
#include <ctype.h>
#include <vector>
using namespace std;


#include "bibentry.h"

#include <qstring.h>
#include <qstringlist.h>
#include <qstylesheet.h>
//extern int flag_allow_warnings;

int flag_allow_warnings = 0;

BibEntry::BibEntry()
{
    def = 0;
    req_field = 0;
    opt_field = 0;
    index = -1;
	special = false;
	preamble = false;
	stringMacro = false;
}

BibEntry::BibEntry(const char *n, const char *k)
{
    def = getBibEntryDef(n);

    if (k)
    {
        key = k;
    }
    else
        key = " ";

    if (def)
    {
        special = false;
	preamble = false;
	stringMacro = false;
        req_field = new string[def->nreq];
        opt_field = new string[def->nopt];
        index = -1;
        asStringMacro = new bool[def->nreq+def->nopt];
        for (int i = 0; i < def->nreq+def->nopt; i++)
            asStringMacro[i] = false;
        setField("lockkey", "N");
    }
    /*
      else {
    	  key = n;
    	  string::iterator it = key.begin();
    	  while (it != key.end()) {
    	    *it = toupper(*it);
    	    it++;
    	  }
    	  if (flag_allow_warnings)
    	    cerr << _("Warning ") << key << ": " << _("Undefined entry type ") << key << "\n";
    	  req_field = new string[1];
    	  opt_field = 0;
    	  special = true;
      }
    */
    nextra = 0;
}


BibEntry::BibEntry(BibEntry& bib)
{
    def = bib.def;
    key = bib.key;

    special =  bib.special;
    preamble = bib.preamble;
    stringMacro = bib.stringMacro;
    index = bib.getIndex();

    if (def)
    {
        req_field = new string[def->nreq];
        opt_field = new string[def->nopt];
        asStringMacro = new bool[def->nreq+def->nopt];
    }
    else if (special)
    {
        req_field = new string[1];
        opt_field = 0;
    }
    nextra = 0;
    setFields(bib);
}



BibEntry& BibEntry::operator=(BibEntry &bib)
{
    if (def)
    {
        delete[] req_field;
        delete[] opt_field;
        delete[] asStringMacro;
        extra_field.clear();
        extra_field_string_macro.clear();
    }

    def = bib.def;
    key = bib.key;

    special =  bib.special;
	preamble = bib.preamble;

    if (def)
    {
        req_field = new string[def->nreq];
        opt_field = new string[def->nopt];
        asStringMacro = new bool[def->nreq+def->nopt];
    }

    nextra = 0;
    setFields(bib);
    return(*this);
}


BibEntry::~BibEntry()
{
    if (def)
    {
        if (req_field)
            delete[] req_field;
        if (opt_field)
            delete[] opt_field;
        if (asStringMacro);
        delete[] asStringMacro;
    }
}

const char *BibEntry::getEntryType() const
{
    //    char const *r = const_cast<char*>((def) ? def->name.c_str(): key.c_str());
    char const *r;
    if (def)
    {
        //r = const_cast<char*>(def->name.c_str());
        r = const_cast<char*>(def->name);
    }
    else r=NULL;
    return r;
}

void BibEntry::setEntryType(char *et)
{
    BibEntry bib(et, const_cast<char*>(key.c_str()));

    if (def)
        bib.setFields(*this);
    else
    {
        def = getBibEntryDef(et);
        if (def)
        {
            req_field = new string[def->nreq];
            opt_field = new string[def->nopt];
        }
        else
        {
            req_field = new string[1];
            opt_field = 0;
        }
        nextra = 0;
    }

    *this = bib;
}

const char *BibEntry::getKey() const
{
    const char *r = const_cast<char*>(key.c_str());
    return r;
}

char *BibEntry::getFieldName(int i)
{
    if (!def)
        return 0;

    if (i<0 || i >= getNoFields())
        return 0;

    if (i < def->nreq + def->nopt)
        return (i < def->nreq) ? def->getRequired(i):
               def->getOptional(i - def->nreq);

    i -= def->nreq + def->nopt;

    slist_pair_string::const_iterator it = extra_field.begin();

    for (; i > 0 && it!=extra_field.end(); i--)
    {
        it++;
    }

    return get_globalfield_name(it->first);
}

char * BibEntry::getField(const char *fn)
{
    if (strcmp(fn, "citation key")==0)
        return const_cast<char*>(key.c_str());

    int x=getNoFields();

    for (int i=0; i< x; i++)
    {
        if (strcasecmp(getFieldName(i), fn)==0)
        {
            return getField(i);
        }
    }
    return 0;
}

bool BibEntry::stringMacroIndicator(const char *fieldnamex)
{
    char *fieldname = new char[strlen(fieldnamex)+1];
	for (int j=0; j < (int)(strlen(fieldnamex)); j++)
        fieldname[j] = tolower(fieldnamex[j]);
    fieldname[strlen(fieldnamex)] = 0;

    int i = def->getFieldIdx(fieldname);
    if (i>=0)
    {

        return asStringMacro[i];
    }
    else
    {
        if (flag_allow_warnings)
            cerr << "Warning " << key << " " << fieldname
            << " does not exist in the entry type <" << getEntryType() << ">\n";
        slist_pair_bool::const_iterator it = extra_field_string_macro.begin();
        while(it!=extra_field_string_macro.end())
        {
            char *fn = get_globalfield_name(it->first);
            if (strcmp(fieldname, fn) == 0)
            {
                break;
            }
            it++;

        }
        return it->second;

    }
}



char * BibEntry::getField(int i)
{
    if (!def)
        return (i==0) ?  const_cast<char*>(req_field[i].c_str()) : 0;

    if (i<0 || i >= getNoFields())
        return 0;
    if (i < def->nreq + def->nopt)
    {
        return (i < def->nreq) ? const_cast<char*>(req_field[i].c_str()) :
               const_cast<char*>(opt_field[i - def->nreq].c_str());
    }
    i -= def->nreq + def->nopt;
    slist_pair_string::const_iterator it = extra_field.begin();
    for (; i > 0 && it!=extra_field.end(); i--)
        it++;
    return const_cast<char*>(it->second.c_str());
}

char *BibEntry::getReqField(int i)
{
    return (i < def->nreq) ? const_cast<char*>(req_field[i].c_str()) : 0;
}


char *BibEntry::getOptField(int i)
{
    return (i < def->nopt) ? const_cast<char*>(opt_field[i].c_str()) : 0;
}

char *BibEntry::getExtField(int i)
{
    slist_pair_string::const_iterator it = extra_field.begin();

    for (; i > 0 && it!=extra_field.end(); i--)
    {
        it++;
    }

    if (it!=extra_field.end())
        return const_cast<char*>(it->second.c_str());

    return 0;
}


void BibEntry::setField(int i, char *field)
{
    if (i < 0 || i >= getNoFields())
        return;

    if (!def && req_field)
    {
        req_field[0] = field;
        return;
    }


    //    cerr << "sField " << key << ":" << i << ": " << field << "\n";
    if (i < def->nreq)
    {
        req_field[i] = field;
    }
    else
    {
        if (i < def->nreq + def->nopt)
        {
            opt_field[i - def->nreq] = field;
        }


        else
        {
            i -= def->nreq + def->nopt;

            slist_pair_string::iterator it = extra_field.begin();

            for (; i > 0 && it!=extra_field.end(); i--)
            {
                it++;
            }
            it->second = field;
        }

    }
}

void BibEntry::setField(char *fieldnamex, char *field)
{
    //Convert fieldname to lowercaseless_findBibEntryBibEntry
    char *fieldname = new char[strlen(fieldnamex)+1];
	for (int j=0; j < (int) (strlen(fieldnamex) ); j++)
        fieldname[j] = tolower(fieldnamex[j]);
    fieldname[strlen(fieldnamex)] = 0;
    if (!def && req_field)
    {
        req_field[0] = field;
        return;
    }

    int i = def->getFieldIdx(fieldname);

    if (i>=0)
    {
        setField(i, field);
        //	cerr << key << ":Field " << fieldname << "|" << field << "\n";
    }
    else
    {
        if (flag_allow_warnings)
            cerr << "Warning " << key << ": Ignored field \"" << fieldname
            << "\" in the entry type <" << getEntryType() << ">.\n";

        // does exist already this name?
        int id = set_globalfield_name(fieldname);
        nextra++;
        extra_field.push_front(pair_string(id, field));

    }
}


void BibEntry::setStringMacroIndicator(const char *fieldnamex, bool s)
{
    //Convert fieldname to lowercaseless_findBibEntryBibEntry
    char *fieldname = new char[strlen(fieldnamex)+1];
	for (int j=0; j < (int) (strlen(fieldnamex) ); j++)
        fieldname[j] = tolower(fieldnamex[j]);
    fieldname[strlen(fieldnamex)] = 0;

    int i = def->getFieldIdx(fieldname);

    if (i>=0)
    {
        setStringMacroIndicator(i, s);
    }
    else
    {
        if (flag_allow_warnings)
            cerr << "Set string warning " << key << ": Ignored field \"" << fieldname
            << "\" in the entry type <" << getEntryType() << ">.\n";
        int id = set_globalfield_name(fieldname);
        extra_field_string_macro.push_front(pair_bool(id, s));
    }
}

/*
void BibEntry::setFields(BibEntry &entry)
{
	clearExtraField();
	for (int i=0; i<entry.getNoFields(); i++)
    {
        string s = entry.getField(i);
        //if (!s.empty()) {
        setField(entry.getFieldName(i), const_cast<char*>(s.c_str()));
        int j = def->getFieldIdx(entry.getFieldName(i));
        if (j >= 0)
            asStringMacro[j] = entry.stringMacroIndicator(i);
		else
			setStringMacroIndicator(entry.getFieldName(i), entry.stringMacroIndicator(entry.getFieldName(i)) );
        //}
    }
}
*/


void BibEntry::setFields(BibEntry &entry)
{
    clearExtraField();
    for (int i=0; i<(entry.getNoReqFields() + entry.getNoOptFields()); i++)
    {
        int j = def->getFieldIdx(entry.getFieldName(i));
        if (j >= 0)
        {
            string s = entry.getField(i);
            setField(entry.getFieldName(i), const_cast<char*>(s.c_str()));
            asStringMacro[j] = entry.stringMacroIndicator(i);
        }
    }

    for (int i = 0; i < entry.getNoExtraFields(); i++)
    {
        int j = i+entry.getNoReqFields() + entry.getNoOptFields();
        string s = entry.getField(j);
        setField(entry.getFieldName(j), const_cast<char*>(s.c_str()));
        setStringMacroIndicator(entry.getFieldName(j), entry.stringMacroIndicator(entry.getFieldName(j)) );

    }


}


/** No descriptions */
int BibEntry::createKey(const char *connectingString)
{
    char *fn = getField("lockkey");
    if (fn[0] == 'Y')
        return -1;

    char *authors = getField("author");

    if (!authors || strlen(authors) == 0)
    {
        //if (strlen(authors)==0){
        authors = getField("editor");
    }
    if (authors&&strlen(authors) > 0)
    {
        QString firstAuthor = (QString(authors).stripWhiteSpace()).section( "and", 0, 0 );
        firstAuthor = firstAuthor.stripWhiteSpace();
        QString firstAuthorSurname;
        if (firstAuthor.contains(',') )
        {
            firstAuthorSurname = firstAuthor.section( ',', 0, 0 );

        }
        else
            firstAuthorSurname = firstAuthor.section(' ', -1, -1 );

        char *st = const_cast<char*>(firstAuthorSurname.ascii());



        key = st;
        key = key + connectingString;
        key = key + getField("year");

        return 1;
    }
    return 0;
}

/** No descriptions */
int BibEntry::getIndex()
{
    return index;
}
/** No descriptions */
void BibEntry::setIndex(int i)
{
    index = i;
}


int compareBibEntry(BibEntry *entry1, BibEntry *entry2, int fields, bool compare_key, bool compare_lockkey)
{
    //return 1 if different, 0 if identical
    QString st1, st2;

    st1 = QString(entry1->getEntryType() ).lower();
    st2 = QString(entry2->getEntryType() ).lower();

    if (st1 != st2)
        return 1;

    char* entryTypeName = const_cast<char*>(entry1->getEntryType());
    BibEntryDef *entryType = getBibEntryDef(entryTypeName);

    if (compare_key)
    {
        st1 = QString(entry1->getKey()).lower();
        st2 = QString(entry2->getKey()).lower();
        if (st1 != st2)
            return 1;
    }

    for (int i=0; i < entryType->nreq; i++)
    {

        st1 = (QString(entry1->getField( entryType->getRequired(i) )).lower()).simplifyWhiteSpace();
        st2 = (QString(entry2->getField( entryType->getRequired(i) )).lower() ).simplifyWhiteSpace();
		if (QString(entryType->getRequired(i) ) == QString("author") || QString(entryType->getRequired(i) ) == QString("editor") ) {
			st1 = reformatAuthors(st1, 0);
			st2 = reformatAuthors(st2, 0);	 
		}
        if (st1 != st2)
            return 1;
    }

    if (fields == 1 || fields==2)
    {
        for (int i=0; i < entryType->nopt; i++)
        {
            st1 = (QString(entry1->getField( entryType->getOptional(i) ) ).lower() ).simplifyWhiteSpace();
            st2 = (QString(entry2->getField( entryType->getOptional(i) ) ).lower() ).simplifyWhiteSpace();

			if (QString(entryType->getOptional(i) ) == QString("author") || QString(entryType->getOptional(i) ) == QString("editor") ) {
				st1 = reformatAuthors(st1, 0);
				st2 = reformatAuthors(st2, 0);	 
			}

            if ( QString(entryType->getOptional(i)).lower() == QString("lockkey") )
            {
                if (compare_lockkey)
                {
                    if (st1 != st2)
                        return 1;
                }
            }

            else
            {
                if (st1 != st2)
                    return 1;
            }
        }

    }

    if (fields == 2)
    {
        if (entry1->getNoExtraFields() != entry2->getNoExtraFields() )
            return 1;
        else
        {
            for (int i = 0; i < entry1->getNoExtraFields(); i++)
            {
                st1 = (QString(entry1->getField( entryType->nreq + entryType->nopt + i) ).lower() ).simplifyWhiteSpace();
                st2 = (QString(entry2->getField(entry1->getFieldName(entryType->nreq + entryType->nopt + i) ) ).lower() ).simplifyWhiteSpace();
				
				if (st1 != st2)
                    return 1;
            }

        }

    }
    return 0;
}


void processAuthor(QString st, QString &surname, QString &givenname)
{
    QString author = (st.simplifyWhiteSpace());
    
/*    author = author.lower();
    QChar c = author.at(0);
    author = author.replace(0, 1, c.upper());

    int i = 0;
    while(i!=-1)
    {
            i=author.find(QString(" "), i+1);
            c = author.at(i+1);
            author = author.replace(i+1, 1, c.upper());
    }
    
    i = 0;
    while(i!=-1)
    {
            i=author.find(QString("-"), i+1);
            c = author.at(i+1);
            author = author.replace(i+1, 1, c.upper());
    }
*/    
    if (author.contains(',') )
    {
        surname = author.section( ',', 0, 0 );
        givenname = author.section( ',', 1, 1 );
    }
    else
    {
        surname = author.section(' ', -1, -1);
        givenname = author.section(' ', -100, -2);
    }
    surname = surname.simplifyWhiteSpace();
    givenname = givenname.simplifyWhiteSpace();

}


QString reformatAuthors(QString st, int numAuthorsDisplayed)
{
    QStringList authors = QStringList::split( " and ", st );
    QString formatedAuthors=QString("");
    bool first = true;
    int num=0;
    for ( QStringList::Iterator it = authors.begin(); it != authors.end(); ++it )
    {
        QString author = (*it);

//        author = (author.simplifyWhiteSpace()).lower();
		author = author.simplifyWhiteSpace();
        QString surname, givenname;
        processAuthor(author, surname, givenname);

		if (givenname.isEmpty())
			author = surname;
		else author = surname + QString(", ") + givenname;
		
/*        author = author.lower();
        QChar c = author.at(0);
        author = author.replace(0, 1, c.upper());

        int i = 0;
        while(i!=-1)
        {
            i=author.find(QString(" "), i+1);
            c = author.at(i+1);
            author = author.replace(i+1, 1, c.upper());
        }
*/
        if (first)
            formatedAuthors = author;
        else
            formatedAuthors = formatedAuthors + " and " + author;
        first = false;
        num++;
        if (numAuthorsDisplayed > 0)
        {
			if (numAuthorsDisplayed >= (int) (authors.count()) && num == (int) (authors.count()) )
                break;
            else if (numAuthorsDisplayed == num)
            {
                formatedAuthors = formatedAuthors + " et al";
                break;
            }
        }
    }
    return formatedAuthors;
}




QString entryToString(BibEntry *e, int fieldDelimiter)
{
    QString ld, rd;
    switch (fieldDelimiter)
    {
    case 1:
        rd = ld="\"";
        break;
    case 2:
        ld = "(";
        rd = ")";
        break;
    default:
        ld = "{";
        rd = "}";
        break;
    }
    QString st;
    st=st+"@"+e->getEntryType()+"{"+e->getKey();
    for (int j=0; j < e-> getNoFields(); j++)
    {
        if ( !((QString(e->getField(j)) ).simplifyWhiteSpace() ).isEmpty() )
        {
            if (e->stringMacroIndicator(e->getFieldName(j)) )
                st=st+",\n\t"+e->getFieldName(j)+"="+e->getField(j);
            else
                st=st+",\n\t"+e->getFieldName(j)+"="+QString(ld)+e->getField(j)+QString(rd);
        }

    }
    st=st+"\n}\n";
    return st;
}

QString detailedEntry(BibEntry *bibItem){
	QString st;
	st = QString("<font color=\"#004faf\"><B>Reference Type:</B></font><BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
	st.append(bibItem->getEntryType());
        //st.append("<BR></pre>");
	st.append("<BR>");
	char fn[250];
	for (int i=0; i < bibItem->getNoFields(); i++)
	{
		strcpy(fn, bibItem->getFieldName(i));
		fn[0] = toupper(fn[0]);
		QString fieldName = QString(bibItem->getFieldName(i));
		if (fieldName.compare("url"))
		{
                //st.append("<pre>");
			st.append("<font color=\"#004faf\">");
			st.append("<B>");
			st.append(fn);
			st.append("</B>:");
			st.append("</font><BR>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
			st.append(QStyleSheet::escape(QString(bibItem->getField(i)).simplifyWhiteSpace()) );
			st.append("<BR>");
                //st.append("</pre>");
		}
	}
	st.append("<font color=\"#004faf\"><B>URL:</B></font> ");
	st.append("<a href=\"");
	st.append(bibItem->getField("url"));
	st.append("\">");
	st.append(bibItem->getField("url"));
	st.append("</a><BR>");
	
	return st;
	
}

QString formatEntry(BibEntry *bibItem, bool richText)
{
    QString st;
    QString tmp;
    if(strcmp(bibItem->getEntryType(), "article")==0)
    {
        tmp = QString(bibItem->getField("author")).simplifyWhiteSpace();
		if (!tmp.isEmpty()){
			tmp = reformatAuthors( tmp, 0);
			st.append(tmp);
		}
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("\"");
            st.append(tmp);
            st.append("\"");
        }
        tmp = QString(bibItem->getField("journal")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            if (richText)
                st.append("<I>");
            st.append(tmp);
            if (richText) st.append("</I>");
        }

        tmp = QString(bibItem->getField("volume")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("vol. ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("number")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("no. ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("pages")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("pp. ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("year")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }
        if (!st.isEmpty())
            st.append(".");

    }

    else if(strcmp(bibItem->getEntryType(), "inproceedings")==0 ||strcmp(bibItem->getEntryType(), "incollection")==0 )
    {
        tmp = QString(bibItem->getField("author")).simplifyWhiteSpace();
		if (!tmp.isEmpty()){
			tmp = reformatAuthors( tmp, 0);
            st.append(tmp);
		}
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("\"");
            st.append(tmp);
            st.append("\"");
        }

        tmp = QString(bibItem->getField("booktitle")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("in ");
            if (richText) st.append("<I>");
            st.append(tmp);
            if (richText) st.append("</I>");
        }

        tmp = QString(bibItem->getField("volume")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("vol. ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("number")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("no. ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("year")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("pages")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("pp. ");
            st.append(tmp);
        }

        if (!st.isEmpty())
            st.append(".");
    }

    else if(strcmp(bibItem->getEntryType(), "book")==0)
    {
        tmp = QString(bibItem->getField("author")).simplifyWhiteSpace();
		if (!tmp.isEmpty()){
			tmp = reformatAuthors( tmp, 0);
			st.append(tmp);
		}
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            if (richText) st.append("<I>");
            st.append(tmp);
            if (richText) st.append("</I>");
        }

        tmp = QString(bibItem->getField("edition")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }


        tmp = QString(bibItem->getField("publisher")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("year")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("chapter")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("ch. ");
            st.append(tmp);
        }
        if (!st.isEmpty())
            st.append(".");
    }

    else if(strcmp(bibItem->getEntryType(), "proceedings")==0)
    {
        tmp = QString(bibItem->getField("editor")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
			tmp = reformatAuthors( tmp, 0);
			st.append(tmp);
            st.append(", Ed.");
        }
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("\"");
            st.append(tmp);
            st.append("\"");
        }

        tmp = QString(bibItem->getField("volume")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("vol. ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("number")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("no. ");
            st.append(tmp);
        }

        QString tmp2 = QString(bibItem->getField("address")).simplifyWhiteSpace();
        if (!tmp2.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp2);
        }

        tmp = QString(bibItem->getField("publisher")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (tmp2.isEmpty())
                st.append(", ");
            else
                st.append(": ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("year")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        if (!st.isEmpty())
            st.append(".");
    }

    else if(strcmp(bibItem->getEntryType(), "phdthesis")==0)
    {
        tmp = QString(bibItem->getField("author")).simplifyWhiteSpace();
		if (!tmp.isEmpty()){
			tmp = reformatAuthors( tmp, 0);
            st.append(tmp);
		}
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("\"");
            st.append(tmp);
            st.append(",\"");
        }
        st.append(" Ph.D. thesis");

        tmp = QString(bibItem->getField("school")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("address")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("year")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }
        st.append(".");


    }

    else if(strcmp(bibItem->getEntryType(), "masterthesis")==0)
    {
        tmp = QString(bibItem->getField("author")).simplifyWhiteSpace();
		if (!tmp.isEmpty()){
			tmp = reformatAuthors( tmp, 0);
            st.append(tmp);
		}
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("\"");
            st.append(tmp);
            st.append(",\"");
        }
        st.append(" Master thesis");

        tmp = QString(bibItem->getField("school")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("address")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        tmp = QString(bibItem->getField("year")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }
        st.append(".");


    }

    else if(strcmp(bibItem->getEntryType(), "inbook")==0)
    {
        tmp = QString(bibItem->getField("author")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
			tmp = reformatAuthors( tmp, 0);
            st.append(tmp);
        }
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("\"");
            st.append(tmp);
            st.append("\"");
        }

    }


    else
    {
        tmp = QString(bibItem->getField("author")).simplifyWhiteSpace();
		if (!tmp.isEmpty()){
			tmp = reformatAuthors( tmp, 0);
            st.append(tmp);
		}
        tmp = QString(bibItem->getField("title")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append("\"");
            st.append(tmp);
            st.append("\"");
        }

        tmp = QString(bibItem->getField("year")).simplifyWhiteSpace();
        if (!tmp.isEmpty())
        {
            if (!st.isEmpty())
                st.append(", ");
            st.append(tmp);
        }

        if (!st.isEmpty())
            st.append(".");

    }
    return st;

}
