/***************************************************************************
                          bibentry.h  -  description
                             -------------------
    begin                : Sat May 24 2003
    copyright            : (C) 2003 by Thach Nguyen
    email                : thach@dragon.thach.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BIBENTRY_H
#define BIBENTRY_H

#include <list>
#include <string>
#include <vector>
#include <slist.h>
#include <stack>
using namespace std;

#include "entrydef.h"
#include <qstring.h>

/**
  *@author Thach Nguyen
  */
typedef pair<int, string> pair_string;


typedef slist<pair_string> slist_pair_string;
//typedef slist<(pair<int, string>)> slist_pair_string;

typedef pair<int, bool> pair_bool;
typedef slist<pair_bool> slist_pair_bool;


typedef pair<string, string> pair_strings;
typedef vector<pair_strings> vector_pair_strings;

class BibEntry
{
public:

    ///
    BibEntry();
    ///
    BibEntry(const char *entrytype, const char *key);
    /// Copy constructor
    BibEntry(BibEntry&);
    ///
    ~BibEntry();

    const char *getEntryType() const;
    const char *getKey() const;
    char *getFieldName(int);

    char *getField(int);
    char *getField(const char*) ;

    char *getReqField(int);
    char *getOptField(int);
    char *getExtField(int);

    int getNoFields()
    {
        return (!def) ? 1:  def->nreq + def->nopt + nextra;
    }

    int getNoReqFields()
    {
        return (!def) ? 1: def->nreq;
    }

    int getNoOptFields()
    {
        return (!def) ? 0: def->nopt;
    }

    int getNoExtraFields()
    {
        return nextra;
    }

    void setEntryType(char *);

    void setKey(char *k) { key = k; }

    void setField(int, char *);

    void setField(char * fieldname, char *);

    void setFields(BibEntry &);

	void clearExtraField(void ) { extra_field.clear(); nextra = 0; extra_field_string_macro.clear();  }

    BibEntry& operator=(BibEntry &);

    bool isSpecial() { return special; }
	bool isPreamble() { return preamble; }
    bool isStringMacro() { return stringMacro; }
	void setPreamble(bool x) { preamble = x; }
    void setStringMacro(bool macro) { stringMacro = macro;}
    int createKey(const char *connectingString="");
    int getIndex();
    void setIndex(int);

    void allowExtraField() { allow_extra = true; }

    bool stringMacroIndicator(int fieldIdx) { return asStringMacro[fieldIdx]; }
    bool stringMacroIndicator(const char* fieldName);
    void setStringMacroIndicator(int idx, bool s) { asStringMacro[idx] = s; }
    void setStringMacroIndicator(const char* fieldName, bool s);
protected:
    string key;
    string *req_field;
    string *opt_field;
    BibEntryDef *def;
    int nextra;
    slist_pair_string extra_field;
	slist_pair_bool extra_field_string_macro;
    bool special;
	bool preamble;
    bool stringMacro;
    bool *asStringMacro;
    bool allow_extra;
    //    friend class BibentryTable;
private: // Private attributes
    int index;
};


int compareBibEntry(BibEntry *e1, BibEntry *e2, int fields=0, bool compare_key = false, bool compare_lockkey = false);
void processAuthor(QString st, QString &surname, QString &givenname);
QString reformatAuthors(QString st, int numAuthorsDisplayed=0);
QString entryToString(BibEntry *entry, int fieldDelimiter = 0);
QString detailedEntry(BibEntry *bibItem);
QString formatEntry(BibEntry *entry, bool richText=true);

#endif
