/***************************************************************************
                          bibentrytable.cpp  -  description
                             -------------------
    begin                : Sun May 25 2003
    copyright            : (C) 2003 by Thach Nguyen
    email                : thach@dragon.thach.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <iostream>
#include <algorithm>
#include <stdlib.h>
using namespace std;

#include "config.h"
#include "bibentrytable.h"


#include <ctime>

bool use_braces = true;

vector<BibEntry *> BibentryTable::entryTab;
vector<BibEntry *> BibentryTable::stringMacroTab;
extern vector<string> fieldNameTab;

int caseless_find(const char *haystack,char *needle, bool caseless);

void print_entry(FILE *fp, BibEntry *e, int fieldDelimiter)
{
    int i;
    char ld, rd;
    switch (fieldDelimiter)
    {
    case 1:
        rd = ld='\"';
        break;
    case 2:
        ld = '(';
        rd = ')';
        break;
    default:
        ld = '{';
        rd = '}';
        break;
    }

    if (e->isSpecial())
    {
        fprintf(fp, "@%s{\n", e->getKey());
        fprintf(fp, "%s\n}\n\n", e->getField(0));
        return;
    }

    if (e->isStringMacro())
    {

        fprintf(fp, "@string{%s=%c", e->getKey(),ld);
        fprintf(fp, "%s%c}\n\n", e->getField(0), rd);
        return;
    }

    fprintf(fp, "@%s{%s,\n", e->getEntryType(), e->getKey());

    for (i=0; i < e-> getNoFields(); i++)
    {
        if (e->getField(i)[0] >= ' ')
        {


            if (e->stringMacroIndicator(e->getFieldName(i)))
            {
                fprintf(fp, "\t%s = %s", e->getFieldName(i), e->getField(i));
            }
            else
                fprintf(fp, "\t%s = %c%s%c", e->getFieldName(i), ld, e->getField(i), rd);
            if (i < e->getNoFields()-1)
                fprintf(fp, ",\n");
        }
    }
    fprintf(fp, "\n}\n\n");
}



BibentryTable::BibentryTable()
{
    dbname = "none";
}

BibentryTable::~BibentryTable()
{
    clear();
}

void BibentryTable::clear()
{
    for (int i=0; i < (signed int)entryTab.size(); i++)
    {
        delete entryTab[i];
    }
    entryTab.erase(entryTab.begin(), entryTab.end());
    //selected_entry = -1;
    //  commands.clear();
}

void BibentryTable::clearStringMacro()
{
    for (int i=0; i < (signed int)stringMacroTab.size(); i++)
    {
        delete stringMacroTab[i];
    }
    stringMacroTab.erase(stringMacroTab.begin(), stringMacroTab.end());
    //selected_entry = -1;
    //  commands.clear();
}

BibEntry *BibentryTable::get_entry(int i)
{
    if (0 <= i && i < (signed int)entryTab.size())
    {
        return entryTab[i];
    }
    return 0;
}

BibEntry *BibentryTable::get_stringMacro(int i)
{
    if (0 <= i && i < (signed int)stringMacroTab.size())
    {
        return stringMacroTab[i];
    }
    return 0;
}

int BibentryTable::readBibfile(char *name, bool import)
{
    BibEntry *bib;
    FILE *inf = fopen(name, "r");

    if (!inf)
        return -1;

    int i = 0;
    if (inf)
    {
        //        while ((bib=read_entry(inf, import)))
        //       {
        /*	    if (bib->isSpecial()) {
        	      new_command(bib->getKey(), bib->getField(0));
        	    } else
        */
        while(!feof(inf))
        {
            bib=read_entry(inf, import);
            if (bib)
            {
                if (bib->isPreamble() )
                {
                    preamble = QString(bib->getField(0) );
                    delete bib;
                }

                else if (bib->isStringMacro() )
                {
                    stringMacroTab.push_back(new BibEntry(*bib));
                    delete bib;
                }
                else
                {
                    bib->setIndex(i);
                    new_entry(*bib);
                    delete bib;
                    i++;
                }
            }
        }
        fclose(inf);
    }


    string aux = name;
    dbname = "";
    for (unsigned int i = aux.rfind('/')+1; i < aux.rfind('.') && i < strlen(name); i++)
    {
        dbname += name[i];
    }
    return 0;
}


void BibentryTable::new_entry(BibEntry &bib)
{
    entryTab.push_back(new BibEntry(bib));
}

void BibentryTable::new_entry(BibEntry* bib)
{
    entryTab.push_back(bib);
}

void BibentryTable::new_stringMacro(BibEntry* bib)
{
    stringMacroTab.push_back(bib);
}



int BibentryTable::writeBibfile(const char *name, int fieldDelimiter)
{
    int i = 0;
    BibEntry *bib;
    FILE *outf = fopen(name, "w");

    if (!outf)
    {
        cerr << "Cannot open file for saving " << name<<"\n";
        return -1;
    }
    //  print_commands(outf, this);
    fprintf(outf, "#Created by Kbib version %s\n", VERSION);
    time_t curr = time(0);
    fprintf(outf, "#Last modified: %s\n\n", ctime(&curr) );

    if (!preamble.isEmpty())
    {
        fprintf(outf, "@preamble{%s}\n\n", preamble.ascii());
    }

    while ((bib=get_stringMacro(i++)))
    {
        print_entry(outf, bib, fieldDelimiter);
    }

    i = 0;
    while ((bib=get_entry(i++)))
    {
        print_entry(outf, bib, fieldDelimiter);
    }
    fclose(outf);
    return 0;
}


/** Only remove the table vector, don't delete the entry contents */
void BibentryTable::removeEntryTab()
{
    entryTab.erase(entryTab.begin(), entryTab.end());
}

void BibentryTable::removeStringMacroTab()
{
    stringMacroTab.erase(stringMacroTab.begin(), stringMacroTab.end());
}


void BibentryTable::createKeys(const char *connectingString)
{
    string fkey,tmp_fkey;
    char dletter='a';
    int oopscount=0;
    int idx;
    char *fn;
    char *fm;
    bool fix_key = false;
    for (int i = 0; i< (signed int) entryTab.size(); i++)
    {
        BibEntry *entry = entryTab[i];
        fn = entry->getField("lockkey");
        if (fn[0]!='Y')
        {
            entry->createKey(connectingString);
            dletter='a';
            fkey= entry->getKey();
            tmp_fkey = fkey;
            entry->setKey("");

            while(1)
            {
                idx = searchKey(const_cast<char*>(tmp_fkey.c_str()));

                if (idx < 0)
                    fix_key = false;

                else if (idx < i)
                    fix_key = true;

                else
                {

                    while(1)
                    {

                        fm = entryTab[idx]->getField("lockkey");
                        if (fm[0] == 'Y')
                        {
                            fix_key = true;
                            break;
                        }
						if (fm[0] != 'Y' && idx == ((int)(entryTab.size())-1))
                        {
                            fix_key = false;
                            break;
                        }

						if (fm[0] != 'Y' && idx < ((int)(entryTab.size())-1))
                        {
                            int tmp_idx = searchKey(const_cast<char*>(tmp_fkey.c_str()), idx+1);
                            idx = tmp_idx;
                        }
                        if (idx < 0)
                        {
                            fix_key = false;
                            break;
                        }
                    }
                }

                if (!fix_key)
                {
                    entry->setKey(const_cast<char*>(tmp_fkey.c_str()));
                    break;
                }
                else
                {
                    tmp_fkey = fkey;
                    for(int j=0;j<oopscount;j++)
                        tmp_fkey+='x';
                    tmp_fkey+=dletter;
                    dletter++;
                    if (dletter=='z')
                    {
                        oopscount++;
                        dletter='a';
                    }
                }
            }
        }
    }
}


int BibentryTable::searchKey(const char *s, int start)
{
    int found = -1;
    //string skey = s;
    QString skey = QString(s).lower();
    for (int fn=start; fn < (signed int)entryTab.size(); fn++)
    {
        BibEntry *entry = entryTab[fn];
        //string key = entry->getKey();
        QString key = QString(entry->getKey()).lower();
        if (key == skey)
        {
            found = fn;
            break;
        }
    }
    return found;
}



//Do I need this function???????
void BibentryTable::createKey(int i, const char *connectingString)
{
    string fkey,tmp_fkey;
    char dletter='a';
    int oopscount=0;
    int idx;
    char *fn;
    char *fm;
    bool fix_key = false;

    if (i < 0 || i >= (signed int) entryTab.size() )
    {
        return;
    }

    BibEntry *entry = entryTab[i];
    fn = entry->getField("lockkey");
    if (fn[0]!='Y')
    {
        entry->createKey(connectingString);
        dletter='a';
        fkey= entry->getKey();
        tmp_fkey = fkey;
        entry->setKey("");

        while(1)
        {
            idx = searchKey(const_cast<char*>(tmp_fkey.c_str()));

            if (idx < 0)
                fix_key = false;

            else if (idx < i)
                fix_key = true;

            else
            {

                while(1)
                {


                    fm = entryTab[idx]->getField("lockkey");
                    if (fm[0] == 'Y')
                    {
                        fix_key = true;
                        break;
                    }
					if (fm[0] == 'N' && idx == ((int)(entryTab.size())-1))
                    {
                        fix_key = false;
                        break;
                    }

					if (fm[0] == 'N' && idx < ((int)(entryTab.size())-1))
                    {
                        idx = searchKey(const_cast<char*>(tmp_fkey.c_str()), idx+1);

                    }
                    if (idx < 0)
                    {
                        fix_key = false;
                        break;
                    }
                }
            }

            if (!fix_key)
            {
                entry->setKey(const_cast<char*>(tmp_fkey.c_str()));
                break;
            }
            else
            {
                tmp_fkey = fkey;
                for(int j=0;j<oopscount;j++)
                    tmp_fkey+='x';
                tmp_fkey+=dletter;
                dletter++;
                if (dletter=='z')
                {
                    oopscount++;
                    dletter='a';
                }
            }
        }
    }
}


/*
void BibentryTable::createKey(int i)
{
        string fkey;
        char dletter='a';
        int oopscount=0;
        int idx;
        if (i < 0 || i >= (signed int) entryTab.size() ) {
                return;
        }
 
        BibEntry *entry = entryTab[i];
        entry->createKey();
        dletter='a';
        fkey= entry->getKey();
        idx = searchKey(const_cast<char*>(fkey.c_str()));
        if (idx!=i) {
                while(1) {
                        fkey= entry->getKey();
                        for(int j=0;j<oopscount;j++)
                                fkey+='x';
                        fkey+=dletter;
                        idx = searchKey(const_cast<char*>(fkey.c_str()));
                        if (idx<0) {
                                entry->setKey(const_cast<char*>(fkey.c_str()));
                                break;
                        } else {
                                dletter++;
                                if (dletter=='z') {
                                        oopscount++;
                                        dletter='a';
                                }
                        }
 
                }
        }
 
}
*/

int caseless_find(const char *haystack,char *needle, bool caseless)
{
    int i,j,flaws;
    int lh,ln;
    int skip_braces=1,dyn_alloc=0;
    char *aux,*p;
    char prebuf[256];

    lh=strlen(haystack);
    ln=strlen(needle);

    if ((lh==0)&&(ln==0))
        return 1;
    if (lh==0)
        return 0;

    if (strchr(needle,'{'))
        skip_braces=0;
    if (strchr(needle,'}'))
        skip_braces=0;

    if (lh>255)
    {
        aux=(char *)malloc(lh+1);
        dyn_alloc=1;
    }
    else
        aux=prebuf;

    if (skip_braces)
    {
        memset(aux,0,lh+1);
        p=aux;
        for(i=0;i<lh;i++)
        {
            if ((haystack[i]=='{')||(haystack[i]=='}'))
                continue;
            *(p++)=haystack[i];
        }
        lh=strlen(aux);
    }
    else
    {
        strcpy(aux,haystack);
    }

    lh=strlen(aux);

    if (ln>lh)
    {
        if (dyn_alloc)
            free(aux);
        return 0;
    }

    for(i=0;i<(lh-ln+1);i++)
    {
        flaws=0;
        for(j=0;j<ln;j++)
        {
            char x, y;
            if (caseless)
            {
                x = toupper(aux[i+j]);
                y = toupper(needle[j]);
            }
            else
            {
                x = aux[i+j];
                y = needle[j];
            }

            //if (toupper(aux[i+j])!=toupper(needle[j])) {
            if (x!=y)
            {
                ++flaws;
                break;
            }
        }
        if (!flaws)
        {
            if (dyn_alloc)
                free(aux);
            return 1;
        }
    }
    if (dyn_alloc)
        free(aux);
    return 0;
}


int BibentryTable::search_string(char *s, int first, int att, int opt)
{
    int fn, last, found = -1;
    bool caseless = 1;

    if ((opt&1))
        caseless = 0;



    if (first >= 0)
    {
        last = entryTab.size();
    }
    else
    {
        last = -first;
        first = 0;
    }

    for (fn=first; fn < last; fn++)
    {
        BibEntry *entry = entryTab[fn];
        char * field;

        /*

        if (att == 1){ //search in author
          field = entry->getField("author");
          if (caseless_find(const_cast<char*>(field.c_str()),s,caseless)) 
           found = fn;
        }
        else if (att == 2) { //search in title
          field = entry->getField("title");
          if (caseless_find(const_cast<char*>(field.c_str()),s,caseless))
           found = fn;
        }
        */

        if (att == 0)
        {
            field = const_cast<char *>(entry->getKey());
            if (field)
            {
                if (caseless_find(field,s,caseless))
                    found = fn;
            }

        }
        else if (att>=0)
        {
            char *field_name = const_cast<char*>(fieldNameTab[att].c_str());
            field = entry->getField(field_name);
            if (field)
            {
                if (caseless_find(field,s,caseless))
                    found = fn;
            }

        }
        else
        { //search in all fields

            for (int i=-1; i < entry->getNoFields(); i++)
            {
                field = (i>=0) ?
                        entry->getField(i) :
                        const_cast<char*>(entry->getKey());
                if (caseless_find(field,s,caseless))
                {
                    found = fn;
                    break;
                }
            }
        }



        if (found >= 0)
            break;
    }

    return found;
}

void BibentryTable::delete_entry(int i)
{
    if (0 <= i && i < ((signed int)entryTab.size()))
    {
        vector<BibEntry *>::iterator it = entryTab.begin();
        for (int k=0; k<i; k++)
            it++;
        delete *it;
        entryTab.erase(it);
    }
}

static string field_to_sort = "citation key";

static
bool lt_fields(BibEntry *e1, BibEntry *e2)
{
    const char *r1,*r2;
    string s1, s2;
    r1 = e1->getField(const_cast<char*>(field_to_sort.c_str()));
    r2 = e2->getField(const_cast<char*>(field_to_sort.c_str()));

    if ((!r1)&&(!r2))
        return 0;
    if (!r1)
        return 1;
    if (!r2)
        return 0;

    s1=r1;
    s2=r2;

    return (s1 < s2);
}

static
bool gt_fields(BibEntry *e1, BibEntry *e2)
{
    const char *r1,*r2;
    string s1, s2;
    r1 = e1->getField(const_cast<char*>(field_to_sort.c_str()));
    r2 = e2->getField(const_cast<char*>(field_to_sort.c_str()));

    if ((!r1)&&(!r2))
        return 0;
    if (!r1)
        return 1;
    if (!r2)
        return 0;

    s1=r1;
    s2=r2;

    return (s1 > s2);
}

void BibentryTable::sort(int ascending, const char *s)
{
    field_to_sort = s;
    ::sort(entryTab.begin(), entryTab.end(),
           ascending ? lt_fields : gt_fields);
    updateEntryIndices();
}

int BibentryTable::removeDuplicated(int compareSetting)
{
    int numRemoved=0;
    BibEntry *thisEntry, *otherEntry;
//    int numField;
    //  bool duplicated = false;
    int listSize = entryTab.size();
    for (int i =0; i < listSize-1; i++)
    {
        thisEntry = entryTab[i];

        for (int j = i+1; j < listSize; j++)
        {
            otherEntry = entryTab[j];

            if (compareBibEntry( thisEntry, otherEntry, compareSetting) == 0)
            {
                delete_entry(j);
                numRemoved++;
                listSize--;
                j--;
            }


        }

    }
    if (numRemoved)
    {
        updateEntryIndices();
    }
    return  numRemoved;


}

void BibentryTable::updateEntryIndices(void)
{
    int listSize = entryTab.size();
    for (int i =0; i <= listSize-1; i++)
    {
        entryTab[i]->setIndex(i);
    }

}

void BibentryTable::change_entry(int index, BibEntry *entry)
{
	if (index < 0 || index >= (int)(entryTab.size()))
        return;

    delete entryTab[index];
    entryTab[index] = entry;
    entryTab[index]->setIndex(index);
}
