/***************************************************************************
                          binfile.cpp  -  description
                             -------------------
    begin                : Sun May 25 2003
    copyright            : (C) 2003 by Thach Nguyen
    email                : thach@dragon.thach.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bibfile.h"


// avoid recognizing characters >127 (negative when c is signed)
// as control cahrs
int lt_space(char c)
{
    return( (c>=0) && (c<=' ') );
}


int ispseudoalpha(char c)
{
    if (isalpha(c)) return 1;
	if (isdigit(c)) return 1;
    return ((c=='-')||(c=='_'));
}


// Read everything inside delimiters
char *read_field(FILE *inf, char &c, bool &asStringMacro)
{
    asStringMacro = false;
    int i=0, braceCount = 0, quoteCount = 0, parenCount=0;
    bool wasslash = false, is_brace, is_quote, is_paren, no_delim;
    static char buf[20000];

    buf[0] = '\0';

    // Eat spaces
    while (lt_space(c) && !feof(inf)) c=getc(inf);

    is_brace = bool(c == '{');
    is_quote = bool(c == '"');
    is_paren = bool(c == '(');
    no_delim = bool(!is_brace && !is_quote && !is_paren);

    while (!feof(inf))
    {
        if ((c==',' || c=='\n') && (no_delim || (is_brace && braceCount==0) ||
                                    (is_quote && quoteCount==0) || (is_paren && parenCount==0) ))
        {
            buf[i] = 0;
            break;
        }

        if (c=='{') braceCount++;

        if (c=='}')
        {
            braceCount--;
            //if ((braceCount<0)||(braceCount==0 && is_brace))
            if (braceCount==0 && is_brace)
            {
                buf[i] = c;
                /*
                            if (braceCount)
                            {
                                braceCount++;
                                buf[i] = 0;
                            }
                            else
                */
                c=' ';
                break;
            }
        }

        if (c=='"' && !wasslash && is_quote)
        {
            quoteCount = 1 - quoteCount;
            if (quoteCount==0)
            {
                buf[i] = '"';
                break;
            }
        }

        if (c=='(') parenCount++;

        if (c==')')
        {
            parenCount--;
            //if ((parenCount<0)||(parenCount==0 && is_paren))
            if (parenCount==0 && is_paren)
            {
                buf[i] = c;
                /*
                            if (parenCount)
                            {
                                parenCount++;
                                buf[i] = 0;
                            }
                            else
                */
                c=' ';
                break;
            }
        }

        buf[i++] = c;

        // Size Limit
        if (i > 19998) break;

        wasslash = (c == '\\');
        c = getc(inf);
    } // main while

    // Broken file
    if (feof(inf) && braceCount>0) return 0;
    /*
       if (braceCount<0)
       {
           buf[i] = 0;
           cerr << "More } than {, possible error " << &buf[0] << "\n";
           i--;
           ungetc('}',inf);
       }

       if (parenCount<0)
       {
           buf[i] = 0;
           cerr << "More ) than (, possible error " << &buf[0] << "\n";
           i--;
           ungetc(')',inf);
       }
    */
    // Eat right spaces
    while (lt_space(buf[i]) && i>0) i--;

    if ((is_brace && buf[0]=='{' && buf[i]=='}') ||
            (is_quote && buf[0]=='"' && buf[i]=='"') ||
            (is_paren && buf[0]=='(' && buf[i]==')') )
    {
        buf[i]=0;
        asStringMacro = false;
        return &buf[1];
    }
    asStringMacro = true;
    buf[i+1]=0;

    return &buf[0];
}

// Read the whole entry
BibEntry *read_entry(FILE *inf, bool import, bool ignoreUndefinedType, bool ignoredUndefinedField )
{
    bool undefinedType = false;
    unsigned int i;
    char c, name[100], key[100], fieldname[100];
    BibEntry *entry = 0;
    Delimiter delim=BRACE;
    char ld='{',rd='}';
    bool stringMacro = false;

    // Eat space
    do c = getc(inf); while (lt_space(c) && !feof(inf));

    // Eat everything before the @
    while (c!='@' && !feof(inf))
    {
        // skip comments
        if (c=='%') while (!feof(inf) && getc(inf)!='\n');

        c = getc(inf);
    }

    // This is not a BibTeX entry!
    if (c != '@') return 0;

    // Eat everything before the name
    do c = getc(inf); while(!ispseudoalpha(c) && !feof(inf));

    // read the entry type name
    for (i=0; ispseudoalpha(c) && !feof(inf); i++)
    {
        name[i] = c;
        c = getc(inf);
    }
    name[i] = 0;

    char lName[255];

    // Non-case sensitive
    for (i=0; name[i] > ' '; i++)
        lName[i] = tolower(name[i]);
    lName[i] = '\0';
    // Check that the name exist, otherwise it's a special case
    if (!getBibEntryDef(name))
    {
        /*       if (isCommand(name)) {
        	  entry = new BibEntry(name, 0);
        */
        //	char *s = read_field(inf, c);
        //	cerr << "Field = " << s << "\n";
        if (strcmp(lName, "preamble")==0)
        {
            entry = new BibEntry("string", 0);
            entry->setPreamble(true);
            bool dump;
            char *s = read_field(inf, c, dump);
            if (s)
                entry->setField(0, s);
            return entry;
        }

        cerr << "Entry type: " << name << " is not found\n";
        undefinedType = true;
        if (!ignoreUndefinedType)
        {
            if (strcmp(lName, "comment") == 0 )
            {
                undefinedType = true;
                return 0;
            }
            else
            {
                cerr << "Create entry type " << name << "\n";
                new_undefined_entrydef(name);
                undefinedType = false;
            }
        }

    }



    if (strcmp(lName, "string") == 0)
    {
        stringMacro = true;
    }
    delim=BRACE;
    if (import || stringMacro)
    {
        while((c!='{') && !feof(inf) )
        {
            c=getc(inf);
        }
        if (c=='{')
        {
            delim=BRACE;
            rd='}';
        }
        key[0] = 0;
    }
    else
    {
        // Eat everything before the key
        while((c==' ' || c=='{' || c=='(' || c=='\n' || c=='\t' || c=='\r') && !feof(inf))
        {
            c=getc(inf);
            if (c=='{') delim=BRACE;
            if (c=='"') delim=QUOTE;
            if (c=='(') delim=PAREN;
        }

        switch(delim)
        {
        case BRACE: ld='{'; rd='}'; break;
        case QUOTE: ld='"'; rd='"'; break;
        case PAREN: ld='('; rd=')'; break;
        }


        for (i=0; (c!=',' && !feof(inf) && i < 99); i++)
        {
            key[i] = c;
            c = getc(inf);
        }
        key[i] = 0;
        if (feof(inf)) return 0;
        if (c!=',') return 0;
    }

    //Eat right white space
    while (lt_space(key[i]) && i > 0 ) i--;
    key[i+1] = 0;


    if (!undefinedType)
    {
        entry = new BibEntry(name, key);
        entry->setStringMacro(stringMacro);
    }
	
    // Read fields
    do
    {
        // Eat any non-alpha, all field names start with alpha
		do c = getc(inf); while (!ispseudoalpha(c) && c!=rd && !feof(inf));
		for (i=0; ispseudoalpha(c) && !feof(inf); i++)
		{
            //fieldname[i] = tolower(c);
            fieldname[i] = c;
            c = getc(inf);
        }
        fieldname[i] = 0;

        while (lt_space(c) && !feof(inf)) c = getc(inf);
        if (c=='=')
        {
            c=getc(inf);
            bool asStringMacro;
            char *s = read_field(inf, c, asStringMacro);
            if (s)
            {
                //Remove left and right white space
                i = 0;
                while (lt_space(s[i]) && i < strlen(s) ) i++;
                s = &s[i];
                i = strlen(s);
                while (lt_space(s[i]) && i > 0 ) i--;
                s[i+1] = 0;
				//if (stringMacro)
				//	cerr << "\n" << fieldname << " " << s << "\n";
                if (strlen(s) > 0)
                {
                    if (stringMacro)
                    {
                        entry->setKey(fieldname);
                        entry->setField(0, s);
                        //entry->setField("id", fieldname);
                        //entry->setField("value", s);
                    }
                    else
                    {
                        if (!undefinedType)
                        {
                            if (strcmp(fieldname, "kbibnote")==0)
                            {
                                entry->setField("comment", s);
                                entry->setStringMacroIndicator("comment", asStringMacro);
                            }
                            else
                            {
                                entry->setField(fieldname, s);
                                entry->setStringMacroIndicator(fieldname, asStringMacro);
                            }
                        }
                    }
                }
            }
        }

    }
    while (c != rd && !feof(inf));

    return entry;
}


BibEntry* processBibEntryFromString(const char *st, int &last_char,bool ignoreUndefinedType, bool ignoredUndefinedField)
{

    bool undefinedType = false;
    unsigned int i,j;
    char c, name[100], key[100], fieldname[100];
    BibEntry *entry = 0;
    Delimiter delim=BRACE;
    char ld='{',rd='}';
    bool asStringMacro = false;
    bool stringMacro = false;

    // Eat space
    i = 0;

    //        while(i<strlen(st)) {

    do
    {
        c = st[i];
        i++;
    }
    while (lt_space(c) && i<strlen(st));


    // Eat everything before the @
    while (c!='@' && i<strlen(st))
    {
        // skip comments
        if (c=='%')
            while (i<strlen(st) && st[i]!='\n')
            {
                i++;
            }

        c = st[i];
        i++;
    }

    // This is not a BibTeX entry!
    if (c != '@')
    {
        last_char = i-1;
        return 0;
    }

    // Eat everything before the name
    do
    {
        c=st[i];
        i++;

    }
    while(!ispseudoalpha(c) && i<strlen(st));

    // read the entry type name
    for (j=0; ispseudoalpha(c) && i < strlen(st) ; j++)
    {
        name[j] = c;
        c = st[i];
        i++;
    }
    name[j] = 0;

    // Check that the name exist, otherwise it's a special case
    if (!getBibEntryDef(name))
    {
        cerr << "Entry type: " << name << " is not found\n";
        undefinedType = true;
        if (!ignoreUndefinedType)
        {
            cerr << "Create entry type " << name << "\n";
            new_undefined_entrydef(name);
            undefinedType = false;
        }
    }
    // Non-case sensitive
    char lName[255];
    for (j=0; name[j] > ' '; j++)
        lName[j] = tolower(name[j]);
    lName[j] = '\0';

    if (strcmp(lName, "string") == 0)
    {
        stringMacro = true;
    }

    delim=BRACE;

    if (stringMacro)
    {
        while((c!='{') && i<strlen(st) )
        {
            c=st[i];
            i++;
        }
        if (c=='{')
        {
            delim=BRACE;
            rd='}';
        }
        key[0] = 0;
    }
    else
    {
        // Eat everything before the key
        while((c==' ' || c=='{' || c=='(' || c=='\n' || c=='\t' || c=='\r') && i<strlen(st))
        {
            c=st[i];
            i++;
            if (c=='{')
                delim=BRACE;
            if (c=='"')
                delim=QUOTE;
            if (c=='(')
                delim=PAREN;
        }

        switch(delim)
        {
        case BRACE:
            ld='{';
            rd='}';
            break;
        case QUOTE:
            ld='"';
            rd='"';
            break;
        case PAREN:
            ld='(';
            rd=')';
            break;
        }

        for (j=0; (c!=',' && i<strlen(st)); j++)
        {
            key[j] = c;
            c = st[i];
            i++;
        }
        key[j] = 0;
        if (i==strlen(st))
        {
            last_char = i-1;
            return 0;
        }
    }
    if (!undefinedType)
    {
        entry = new BibEntry(name, key);
        entry->setStringMacro(stringMacro);
    }
    //    cerr << "name=" << name << endl;
    //    cerr << "key=" << key << endl;

    // Read fields
    do
    {
        // Eat any non-alpha, all field names start with alpha
        do
        {
            c = st[i];
            i++;
        }
		while (!ispseudoalpha(c) && c!=rd && i<strlen(st));

        for (j=0; ispseudoalpha(c) && j<strlen(st); j++)
        {
            fieldname[j] = tolower(c);
            c= st[i];
            i++;
        }
        fieldname[j] = 0;


        while (lt_space(c) && i<strlen(st))
        {
            c = st[i];
            i++;
        }

        if (c=='=')
        {
            c = st[i];
            i++;
            //char *s = read_field(inf, c);
            char *s;
            // Read everything inside delimiters

            int k=0, braceCount = 0, quoteCount = 0, parenCount=0;
            bool wasslash = false, is_brace, is_quote, is_paren, no_delim;
            char buf[20000];

            buf[0] = '\0';

            // Eat spaces
            while (lt_space(c) && i<strlen(st))
            {
                //c=getc(inf);
                c=st[i];
                i++;
            }

            is_brace = bool(c == '{');
            is_quote = bool(c == '"');
            is_paren = bool(c == '(');
            no_delim = bool(!is_brace && !is_quote && !is_paren);

            while (i<strlen(st))
            {
                if ((c==',' || c=='\n') && (no_delim || (is_brace && braceCount==0) ||
                                            (is_quote && quoteCount==0) || (is_paren && parenCount==0) ))
                {
                    buf[k] = 0;
                    break;
                }

                if (c=='{')
                    braceCount++;

                if (c=='}')
                {
                    braceCount--;
                    //if ((braceCount<0)||(braceCount==0 && is_brace))
                    if (braceCount==0 && is_brace)
                    {
                        buf[k] = c;
                        /*if (braceCount)
                        {
                            braceCount++;
                            buf[k] = 0;
                        }
                        else
                        */
                        c=' ';

                        break;
                    }
                }



                if (c=='"' && !wasslash && is_quote)
                {
                    quoteCount = 1 - quoteCount;
                    if (quoteCount==0)
                    {
                        buf[k] = '"';
                        break;
                    }
                }


                if (c=='(')
                    parenCount++;

                if (c==')')
                {
                    parenCount--;
                    //                    if ((parenCount<0)||(parenCount==0 && is_paren))
                    if (parenCount==0 && is_paren)
                    {
                        buf[k] = c;
                        /*
                        if (parenCount)
                                          {
                                              parenCount++;
                                              buf[k] = 0;
                                          }
                                          else
                        */
                        c=' ';

                        break;
                    }
                }

                buf[k++] = c;

                // Size Limit
                if (k > 19998)
                    break;

                wasslash = (c == '\\');
                c= st[i];
                i++;
            } // main while

            // Broken file
            if (i==strlen(st) && braceCount>0)
            {
                last_char = i-1;
                return 0;
            }
            /*
                     if (braceCount<0)
                     {
                         buf[k+1] = 0;
                         cerr << "More } than {, possible error " << &buf[0] << "\n";
                         //k--;
                         //ungetc('}',inf);
                         //i--;
                         c=st[i-1];
                     }

                     if (parenCount<0)
                     {
                         buf[k+1] = 0;
                         cerr << "More ) than (, possible error " << &buf[0] << "\n";
                         //k--;
                         //ungetc(')',inf);

                         //i--;
                         c=st[i-1];
                     }
            */

            // Eat right spaces
            while (lt_space(buf[k]) && k>0)
                k--;

            if ((is_brace && buf[0]=='{' && buf[k]=='}') ||
                    (is_quote && buf[0]=='"' && buf[k]=='"') ||
                    (is_paren && buf[0]=='(' && buf[k]==')') )
            {
                buf[k]=0;
                s= &buf[1];
                asStringMacro = false;
            }
            else
            {
                buf[k+1]=0;
                s= &buf[0];
                asStringMacro = true;
            }

            if (s)
            {
                if (stringMacro)
                {
                    entry->setKey(fieldname);
                    entry->setField(0, s);
                    //entry->setField("id", fieldname);
                    //entry->setField("value", s);
                }
                else
                {
                    if (!undefinedType)
                    {
                        if (strcmp(fieldname, "kbibnote")==0)
                        {
                            entry->setField("comment", s);
                            entry->setStringMacroIndicator("comment", asStringMacro);
                        }
                        else
                        {
                            entry->setField(fieldname, s);
                            entry->setStringMacroIndicator(fieldname, asStringMacro);
                        }
                    }
                }
            }
        }

    }
    while (c != rd && i<strlen(st));
    last_char = i-1;

    return entry;
}

