/***************************************************************************
                          kbibdoc.cpp  -  description
                             -------------------
    begin                : Sat May 24 16:37:30 EST 2003
    copyright            : (C) 2003 by Thach Nguyen
    email                : thach@dragon.thach.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <fstream>
using namespace::std;

// include files for Qt
#include <qdir.h>
#include <qwidget.h>
#include <qstring.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <ktempfile.h>
// application specific includes
#include "kbibdoc.h"
#include "kbib.h"

#include "kbibeditentrydlg.h"

extern vector<string> entrydefTab;

KBibDoc::KBibDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
    p_view = 0;
    entry_list = new BibentryTable();
    modified = false;
}

KBibDoc::~KBibDoc()
{
    delete entry_list;
    delete  p_view;
}


bool KBibDoc::saveModified()
{
    bool completed=true;

    if(modified)
    {
        KBibApp *win=(KBibApp *) parent();
        int want_save = KMessageBox::warningYesNoCancel(win,
                        i18n("The current file has been modified.\n"
                             "Do you want to save it?"),
                        i18n("Warning"));
        switch(want_save)
        {
        case KMessageBox::Yes:
            if (m_url.fileName() == i18n("Untitled"))
            {
                win->slotFileSaveAs();
            }
            else
            {
                saveDocument(m_url);
            };

            deleteContents();
            completed=true;
            break;

        case KMessageBox::No:
            setModified(false);
            deleteContents();
            completed=true;
            break;

        case KMessageBox::Cancel:
            completed=false;
            break;

        default:
            completed=false;
            break;
        }
    }

    return completed;
}

void KBibDoc::closeDocument()
{
    deleteContents();
}

bool KBibDoc::newDocument()
{
    modified=false;
    m_url.setFileName(i18n("Untitled"));
    p_view->clearListView();
	group_view->clearGroupView();
    entry_list->removeEntryTab();
    entry_list->removeStringMacroTab();
    authorList.clear();
	journalList.clear();
    return true;
}


void KBibDoc::deleteContents()
{
    /////////////////////////////////////////////////
    // TODO: Add implementation to delete the document contents
    /////////////////////////////////////////////////


}


void KBibDoc::addListView(KBibListView *view)
{
    p_view = view;

    //connect(p_view,SIGNAL(signalListChange()), this, SLOT(slotEntryListChange()));
}


void KBibDoc::addGroupView(KBibGroupView *view)
{
	group_view = view;

}



bool KBibDoc::openDocument(const KURL url, bool import)
{
    KTempFile tmpfile;
    tmpfile.setAutoDelete(TRUE);
    QString fn = tmpfile.name();
    //	cerr << "File name " << (url.url()).ascii();
    KBibApp* parent = (KBibApp*) this->parent();
    if(!KIO::NetAccess::download(url, fn, parent))
    {
        return 0;

    }


    char * st =  const_cast<char*>(fn.ascii());

    ifstream tmp_file(st);
    if (!tmp_file)
    {
        return false;
    }

    tmp_file.close();
    entry_list->clear();
    entry_list->clearStringMacro();

    if (entry_list->readBibfile(st, import)==0)
    {
        p_view -> updateListView(entry_list);
		group_view->addBibEntryTable(entry_list);

        //    fileName = filename;
        m_url = url;
        modified=false;

        createAuthorList();
		createJournalList();
		createKeywordList();
        return true;
    }
    else
    {
        return false;
    }
}





bool KBibDoc::saveDocument(const KURL& url)
{
    KBibApp* parent = (KBibApp*) this->parent();
    if(url.isLocalFile())
    {
		if (entry_list->writeBibfile((url.path()).ascii(), (parent->kbibConfigState).fieldDelimiter)==0)
        {
            m_url = url;
            modified=false;
            return true;
        }
        else
        {
            QString msg=QString(i18n("Error in saving file!"));
            KMessageBox::error(parent, msg);
            return false;
        }
    }
    else
    {
        KTempFile tmpfile;
        tmpfile.setAutoDelete(TRUE);
        QString fn = tmpfile.name();
		if (entry_list->writeBibfile(fn.ascii(), (parent->kbibConfigState).fieldDelimiter)==0)
        {
            bool uploaded = KIO::NetAccess::upload(fn, url, parent);
            if(!uploaded)
            {
                QString msg(i18n("Kbib is unable to upload the file - %1.").arg(url.url()));
                KMessageBox::error(parent, msg);
                return false;
            }
            else
            {
                m_url = url;
                modified=false;
                return true;
            }
        }
        return false;
    }

}



int KBibDoc::size(void)
{
    return entry_list->size();
}


void KBibDoc::createKeys(const char *connectingString)
{
    entry_list->createKeys(connectingString);
    p_view->updateCurrentListView();
    modified = true;
    KBibApp *win=(KBibApp *) parent();

    win->slotDocChanged();
}

int KBibDoc::searchKey(const char *st, int start)
{
    return entry_list->searchKey(st, start);
}

BibEntry * KBibDoc::getEntry(int i)
{
    return entry_list->get_entry(i);
}

BibentryTable * KBibDoc::getEntryList(void)
{
    return entry_list;
}



int KBibDoc::search_string(char * st, int first, int att, int opt)
{
    int i = entry_list->search_string(st, first, att, opt);
    p_view->highlightItem(i);
    return i;
}

void KBibDoc::sort(int ascending, QString st)
{
    entry_list->sort(ascending, st.ascii());
    KBibApp *win=(KBibApp *) parent();
    p_view->updateCurrentListView();
    modified = true;
    win->slotDocChanged();
}


int KBibDoc::removeDuplicated()
{
	KBibApp *win = (KBibApp *) parent();
	int compareSetting = (win->kbibConfigState).compareRefSetting;
	int numRemoved = entry_list->removeDuplicated(compareSetting);

    if (numRemoved)
    {
        
        modified = true;
        win->slotDocChanged();
        //p_view->updateCurrentListView();
        p_view->updateListView(entry_list);
        win->slotNumRefMsg();
		if (win->isGroupViewVisible() )
			group_view->createGroupView();
    }
    return numRemoved;
}


int KBibDoc::deleteEntry(int i)
{
    if (i < 0 || i >= this->size() )
    {
        return 0;

    }
    else
    {
        entry_list->delete_entry(i);
        entry_list->updateEntryIndices();

        KBibApp *win = (KBibApp *) parent();
        win->slotNumRefMsg();
        modified = true;
        win->slotDocChanged();
		if (win->isGroupViewVisible() )
			group_view->createGroupView();
        //p_view->updateCurrentListView();

        return 1;

    }

}

int KBibDoc::deleteEntry(vector<int> *i)
{
    std::sort(i->begin(), i->end());
    int numDelete = 0;
    vector<int>::iterator p;
    int j;
    for (p=i->begin(); p!=i->end(); p++)
    {
        j = *p-numDelete;
        if (j >=0 && j < this->size())
        {
            entry_list->delete_entry(j);
            numDelete++;
        }
    }
    entry_list->updateEntryIndices();

    if (numDelete)
    {
        KBibApp *win = (KBibApp *) parent();
        win->slotNumRefMsg();

        modified = true;
        win->slotDocChanged();
		if (win->isGroupViewVisible() )
			group_view->createGroupView();
        //p_view->updateListView(entry_list);
    }

    return numDelete;

}


void KBibDoc::paste(QString str)
{

    if(!str)
        return;
    char *st = const_cast<char*>(str.ascii());
    int last_char = 0;
    while(strlen(st))
    {

        BibEntry *entry = 0;
        entry = processBibEntryFromString(st, last_char);

        if (entry)
        {
            //String Macro
            if (entry->isStringMacro() )
            {
                entry_list->new_stringMacro( entry );
                modified = true;
            }

            else
            {
                //Add entry to the list
                int index = this->size();
                entry->setIndex(index);

                entry_list->new_entry(entry);




                modified = true;

                p_view->addNewItem(entry);
                addToAuthorList(entry);
				addToJournalList(entry);
				addToKeywordList(entry);
				group_view->addEntry( entry );
            }

        }
        st = &st[last_char+1];

    }
    if (modified)
    {
        KBibApp *win = (KBibApp *) parent();
        win->slotDocChanged();
        win->slotNumRefMsg();
    }
}



/*void KBibDoc::paste(QString str)
{
  int i,j;
  char c, name[100], key[100], fieldname[100];
  BibEntry *entry = 0;
  Delimiter delim=BRACE;
  char ld,rd;
 
  //    QClipboard *cb = QApplication::clipboard();
 
 
  //    QString str = cb->text(QClipboard::Clipboard);
 
  if(!str)
    return;
  char *st = const_cast<char*>(str.ascii());
 
  // Eat space
  i = 0;
 
  while(i<strlen(st))
  {
 
    do
    {
      c = st[i];
      i++;
    }
    while (lt_space(c) && i<strlen(st));
 
 
    // Eat everything before the @
    while (c!='@' && i<strlen(st))
    {
      // skip comments
      if (c=='%')
        while (i<strlen(st) && st[i]!='\n')
        {
          i++;
        }
 
      c = st[i];
      i++;
    }
 
    // This is not a BibTeX entry!
    if (c != '@')
      return;
 
    // Eat everything before the name
    do
    {
      c=st[i];
      i++;
 
    }
    while(!ispseudoalpha(c) && i<strlen(st));
 
    // read the entry type name
    for (j=0; ispseudoalpha(c) && i < strlen(st) ; j++)
    {
      name[j] = c;
      c = st[i];
      i++;
    }
    name[j] = 0;
 
    // Check that the name exist, otherwise it's a special case
    if (!getBibEntryDef(name))
    {
      cerr << "Entry type: " << name << " is not found\n";
      return;
    }
 
    delim=BRACE;
 
    // Eat everything before the key
    while((c==' ' || c=='{' || c=='(' || c=='\n' || c=='\t' || c=='\r') && i<strlen(st))
    {
      c=st[i];
      i++;
      if (c=='{')
        delim=BRACE;
      if (c=='"')
        delim=QUOTE;
      if (c=='(')
        delim=PAREN;
    }
 
    switch(delim)
    {
    case BRACE:
      ld='{';
      rd='}';
      break;
    case QUOTE:
      ld='"';
      rd='"';
      break;
    case PAREN:
      ld='(';
      rd=')';
      break;
    }
 
    for (j=0; (c!=',' && i<strlen(st)); j++)
    {
      key[j] = c;
      c = st[i];
      i++;
    }
    key[j] = 0;
    if (i==strlen(st))
      return;
 
    entry = new BibEntry(name, key);
    //    cerr << "name=" << name << endl;
    //    cerr << "key=" << key << endl;
 
    // Read fields
    do
    {
      // Eat any non-alpha, all field names start with alpha
      do
      {
        c = st[i];
        i++;
      }
      while (!isalpha(c) && c!=rd && i<strlen(st));
 
      for (j=0; ispseudoalpha(c) && j<strlen(st); j++)
      {
        fieldname[j] = tolower(c);
        c= st[i];
        i++;
      }
      fieldname[j] = 0;
 
 
      while (lt_space(c) && i<strlen(st))
      {
        c = st[i];
        i++;
      }
 
      if (c=='=')
      {
        c = st[i];
        i++;
        //char *s = read_field(inf, c);
        char *s;
        // Read everything inside delimiters
 
        int k=0, braceCount = 0, quoteCount = 0, parenCount=0;
        bool wasslash = false, is_brace, is_quote, is_paren, no_delim;
        char buf[2000];
 
        buf[0] = '\0';
 
        // Eat spaces
        while (lt_space(c) && i<strlen(st))
        {
          //c=getc(inf);
          c=st[i];
          i++;
        }
 
        is_brace = bool(c == '{');
        is_quote = bool(c == '"');
        is_paren = bool(c == '(');
        no_delim = bool(!is_brace && !is_quote && !is_paren);
 
        while (i<strlen(st))
        {
          if ((c==',' || c=='\n') && (no_delim || (is_brace && braceCount==0) ||
                                      (is_quote && quoteCount==0) || (is_paren && parenCount==0) ))
          {
            buf[k] = 0;
            break;
          }
 
          if (c=='{')
            braceCount++;
 
          if (c=='}')
          {
            braceCount--;
            if ((braceCount<0)||(braceCount==0 && is_brace))
            {
              buf[k] = c;
              if (braceCount)
              {
                braceCount++;
                buf[k] = 0;
              }
              else
                c=' ';
              break;
            }
          }
 
          if (c=='"' && !wasslash && is_quote)
          {
            quoteCount = 1 - quoteCount;
            if (quoteCount==0)
            {
              buf[k] = '"';
              break;
            }
          }
 
          if (c=='(')
            parenCount++;
 
          if (c==')')
          {
            parenCount--;
            if ((parenCount<0)||(parenCount==0 && is_paren))
            {
              buf[k] = c;
              if (parenCount)
              {
                parenCount++;
                buf[k] = 0;
              }
              else
                c=' ';
              break;
            }
          }
 
          buf[k++] = c;
 
          // Size Limit
          if (k > 1998)
            break;
 
          wasslash = (c == '\\');
          c= st[i];
          i++;
        } // main while
 
        // Broken file
        if (i==strlen(st) && braceCount>0)
          return;
 
        if (braceCount<0)
        {
          buf[k] = 0;
          cerr << "More } than {, possible error " << &buf[0] << "\n";
          k--;
          //ungetc('}',inf);
          i--;
          c=st[i-1];
        }
 
        if (parenCount<0)
        {
          buf[k] = 0;
          cerr << "More ) than (, possible error " << &buf[0] << "\n";
          k--;
          //ungetc(')',inf);
 
          i--;
          c=st[i-1];
        }
 
        // Eat right spaces
        while (lt_space(buf[k]) && k>0)
          k--;
 
        if ((is_brace && buf[0]=='{' && buf[k]=='}') ||
            (is_quote && buf[0]=='"' && buf[k]=='"') ||
            (is_paren && buf[0]=='(' && buf[k]==')') )
        {
          buf[k]=0;
          s= &buf[1];
        }
        else
        {
          buf[k+1]=0;
          s= &buf[0];
        }
 
 
 
        if (s)
        {
          entry->setField(fieldname, s);
        }
      }
 
    }
    while (c != rd && i<strlen(st));
 
 
    if (entry==0)
    {
      return;
    }
 
    //Add entry to the list
    int index = this->size();
    entry->setIndex(index);
 
    entry_list->new_entry(entry);
 
    KBibApp *win = (KBibApp *) parent();
 
 
    modified = true;
    win->slotDocChanged();
    p_view->addNewItem(entry);
    win->slotNumRefMsg();
    //p_view->updateListView(entry_list);
 
 
  }
 
 
}
*/





bool KBibDoc::newEntry()
{
    KBibApp* win = (KBibApp *) parent();
    win->slotStatusMsg(i18n("Insert new BibTex entry...") );

    if (entrydefTab.size() < 1)
    {
        KMessageBox::error(win,i18n( "No entry type define!" ) );
        return false;

    }
    BibEntry *entry = editEntry(0);
    if (entry==0)
    {
        win->slotStatusMsg(i18n("Ready") );
        return false;
    }

    //Add entry to the list
	addEntry( entry);
    win->slotStatusMsg(i18n("Ready") );
    return true;

}


void KBibDoc::addEntry( BibEntry *entry, bool ignoreExisting, bool fixDuplicatedKey){
	KBibApp* win = (KBibApp *) parent();

	//Check for existing entry	
	int index = this->size();
	bool insert = true;
	if (ignoreExisting){
		for (int i = 0; i < index; i++){
			BibEntry *e = getEntry(i);
			if (compareBibEntry(e, entry, (win->kbibConfigState).compareRefSetting) == 0){
				insert = false;
				break;
			}
		}
	}
	if (!insert)
		return;
	
	//Add entry to the list
	win->slotStatusMsg(i18n("Insert new BibTex entry...") );
	entry->setIndex(index);
	entry_list->new_entry(entry);
	if (fixDuplicatedKey){
		entry_list->createKey(index, ((win->kbibConfigState).keyConnecting).ascii());	
	}
	

	
    //Add author to the list
	addToAuthorList(entry);
	addToJournalList(entry);
	addToKeywordList(entry);
	modified = true;
	win->slotDocChanged();
    //p_view->updateListView(entry_list);
	p_view->addNewItem(entry);
	group_view->addEntry( entry);
	win->slotStatusMsg(i18n("Ready") );
	win->slotNumRefMsg();
}

bool KBibDoc::modifyEntry(int index)
{
    BibEntry* entry = entry_list->get_entry(index);
    BibEntry* x_entry = editEntry(entry);
    if (x_entry)
    {
        entry->setEntryType(const_cast<char*>(x_entry->getEntryType()));
        entry->setFields(*x_entry);
        entry->setIndex(index);
        entry->setKey(const_cast<char*>(x_entry->getKey()));
        delete x_entry;
        //entry->setField("author", "test");

        //entry_list->change_entry(index, x_entry);

        addToAuthorList(entry);
		addToJournalList(entry);
		addToKeywordList(entry);
        modified = true;
        KBibApp* win = (KBibApp *) parent();
        win->slotDocChanged();
        //p_view->updateListView(entry_list);
		if (win->isGroupViewVisible() )
			group_view->createGroupView();
        return true;
    }
    else
        return false;

}

BibEntry*  KBibDoc:: editEntry(BibEntry * xentry)
{
    BibEntry* entry = xentry;
    bool edit = true;
    KBibApp* win = (KBibApp *) parent();
    while(edit)
    {
        KBibEditEntryDlg* editEntryDlg = new KBibEditEntryDlg(entry, win);
        if (editEntryDlg->exec()==QDialog::Accepted)
        {

            entry = new BibEntry(*(editEntryDlg->getEditedBibEntry()));
            int x = entry->getNoReqFields();
            bool emtyReqFields=false;
            edit = false;
            if (QString(entry->getKey())=="")
                emtyReqFields = true;
            else
            {
                for (int i = 0; i < x; i++)
                {
                    if (QString(entry->getField(i))=="")
                    {
                        //cerr << "emtry fields\n";
                        emtyReqFields = true;
                        break;
                    }
                }
            }

            if (emtyReqFields)
            {

                if((win->kbibConfigState).askReqField)
                {

                    int query = KMessageBox::warningYesNo(win,i18n( "Some required fields are empty.\nDo you want to correct?" ) );
                    if (query==KMessageBox::Yes)
                    {
                        edit =true;
                    }
                }
            }

            if (!edit)
            {
                delete editEntryDlg;
                return entry;
            }


            //    return editEntryDlg->getBibEntry();
        }
        else
        {
            delete editEntryDlg;
            return 0;
        }
    }
}


int KBibDoc::importBibTexFile(const char *fn, bool import, bool notInsertExisting)
{
    int numRefs= 0;
    int numMacros = 0;
    BibEntry *bib;
    FILE *inf = fopen(fn, "r");
    int j = size();
    bool insert = false;
    int listSize;
    BibEntry *thisEntry;
    bool existing = false;
//    int numField;
    if (inf)
    {
        while(!feof(inf))
        {
            bib=read_entry(inf, import);
            if (bib)
            {
                if (bib->isStringMacro() )
                {
                    if (!notInsertExisting)
                    {
                        insert = true;
                    }

                    else
                    {
                        insert = true;
                        listSize = entry_list->stringMacroSize();
                        for (int i =0; i < listSize; i++)
                        {
                            thisEntry = entry_list->get_stringMacro(i);
                            existing = false;

                            if (compareBibEntry( thisEntry, bib, true) == 0)
                            {
                                existing = true;
                            }
                            else
                                existing = false;

                            if (existing)
                            {
                                insert = false;
                                break;
                            }
                        }

                    }

                    if (insert)
                    {
                        entry_list->new_stringMacro(new BibEntry(*bib));
                        numMacros++;
                    }
                    delete bib;
                }

                else
                {

                    if (!notInsertExisting)
                    {
                        insert = true;
                    }
                    else
                    {
                        insert = true;
                        listSize = entry_list->size();
                        for (int i =0; i < listSize; i++)
                        {
                            thisEntry = entry_list->get_entry(i);
                            existing = false;

                            if (compareBibEntry( thisEntry, bib) == 0)
                            {
                                existing = true;
                            }
                            else
                                existing = false;
                            /*
                            if (strcmp(thisEntry->getEntryType(), bib->getEntryType()) == 0)
                                              {
                                                  existing = true;
                                                  numField = thisEntry->getNoFields();
                                                  for (int k = 0; k < numField; k++)
                                                  {
                                                      if (strcmp(thisEntry->getFieldName(k), "lockkey"))
                                                      {

                                                          if (strcmp(thisEntry->getField(k), bib->getField(k)))
                                                          {
                                                              existing = false;
                                                              break;
                                                          }
                                                      }
                                                  }

                                              }
                            */

                            if (existing)
                            {
                                insert = false;
                                break;
                            }
                        }

                    }


                    if (insert)
                    {
                        bib->setIndex(j);
                        j++;
                        entry_list->new_entry(bib);
                        numRefs++;
                        p_view->addNewItem(bib);
						group_view->addEntry( bib);
                    }
                    else
                        delete bib;

                }
            }
        }
        fclose(inf);

    }

    if (numRefs||numMacros)
    {
        KBibApp* win = (KBibApp *) parent();
        if (numRefs)
        {
            win->slotNumRefMsg();
            createAuthorList();
			createJournalList();
			createKeywordList();
        }
        modified = true;
        win->slotDocChanged();
    }
    return numRefs;

}


void KBibDoc::createAuthorList()
{
    authorList.clear();

    int listSize = entry_list->size();
    BibEntry* entry;
    for (int i =0; i < listSize; i++)
    {
        entry = entry_list->get_entry(i);
        addToAuthorList(entry);
    }
    authorList.sort();
}

void KBibDoc::addToAuthorList(BibEntry *entry)
{
    QString authors= QString(entry->getField("author"));
    if (!authors.isEmpty())
        addToAuthorList(authors);
    authors= QString(entry->getField("editor"));
    if (!authors.isEmpty())
        addToAuthorList(authors);
}

void KBibDoc::addToAuthorList(QString st)
{
    QStringList authors = QStringList::split( " and ", st );
    for ( QStringList::Iterator it = authors.begin(); it != authors.end(); ++it )
    {
        QString author = (*it);

		author = (author.simplifyWhiteSpace()); //.lower();

        QString surname, givenname;
        
	processAuthor(author, surname, givenname);

        //author = surname + QString(", ") + givenname;
        /*
	author = author.lower();
        QChar c = author.at(0);
        author = author.replace(0, 1, c.upper());
        //cerr << "\nAuthor = " << author << "\n";

        int i = 0;
        while(i!=-1)
        {
            i=author.find(QString(" "), i+1);
            c = author.at(i+1);
            author = author.replace(i+1, 1, c.upper());
        }
	*/
	if (givenname.isEmpty())
		author = surname;
	else author = surname + QString(", ") + givenname;
	
        if (authorList.findIndex(author) == -1)
        {
            authorList.append(author);
            //cerr << cerr << "\nAuthor = " << author << " added to the list \n";
        }

    }
}

void KBibDoc::createJournalList()
{
	journalList.clear();

	int listSize = entry_list->size();
	BibEntry* entry;
	for (int i =0; i < listSize; i++)
	{
		entry = entry_list->get_entry(i);
		addToJournalList(entry);
	}
	journalList.sort();
}


void KBibDoc::addToJournalList(BibEntry *entry)
{
	QString journal= QString(entry->getField("journal"));
	addToJournalList( journal);
}

void KBibDoc::addToJournalList( QString st)
{
	QString journal = st.simplifyWhiteSpace();
	if (!journal.isEmpty()){
		if (journalList.findIndex(journal) == -1)
		{
			journalList.append(journal);
		}	
	}
}

void KBibDoc::createKeywordList()
{
	keywordList.clear();

	int listSize = entry_list->size();
	BibEntry* entry;
	for (int i =0; i < listSize; i++)
	{
		entry = entry_list->get_entry(i);
		addToKeywordList(entry);
	}
	keywordList.sort();
}


void KBibDoc::addToKeywordList(BibEntry *entry)
{
	QString kws= QString(entry->getField("keywords"));
	addToKeywordList( kws);
}

void KBibDoc::addToKeywordList(QString st)
{
	KBibApp *win=(KBibApp *) parent();
	QStringList kws = QStringList::split( (win->kbibConfigState).keywordSeperator, st );
	for ( QStringList::Iterator it = kws.begin(); it != kws.end(); ++it )
	{
		QString kw = (*it);
		
		kw = kw.simplifyWhiteSpace();
		if (!kw.isEmpty()) {
			if (keywordList.findIndex(kw) == -1)
			{
				keywordList.append(kw);
			}	
		}

	}
}
