//
// C++ Implementation: kbibeditauthors
//
// Description:
//
//
// Author: Thach Nguyen <thach.nguyen@rmit.edu.au>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "kbibeditauthors.h"
#include <klocale.h>
#include <kpushbutton.h>
#include <kiconloader.h>
#include <qlayout.h>
#include <qlistview.h>
#include <qheader.h>
#include <qvbox.h>
#include <qlabel.h>
#include <qtooltip.h>

#include "bibentry.h"
#include "kbibeditentrydlg.h"
#include "kbib.h"
#include "kbibdoc.h"

KBibEditAuthors::KBibEditAuthors(QWidget *parent, const char *name, bool modal)
        : KDialogBase(parent, name, modal, i18n("Edit authors/editors"), Ok|Cancel)
        //KDialogBase(parent, name, modal, QString::null, Ok|Cancel)
{
    QWidget* page = new QWidget(this);
    QVBoxLayout* v = new QVBoxLayout(page, 0, KDialog::spacingHint());

    QHBoxLayout* t = new QHBoxLayout(page, 0, KDialog::spacingHint());
    QLabel *lb = new QLabel("Existing authors in the current document: ", page);
    existingAuthorSelect = new KComboBox(page, "" );

    t->addWidget(lb);
    t->addWidget(existingAuthorSelect);

    lb->setMinimumWidth(100);
    t->setStretchFactor(lb, 0);
    t->setStretchFactor(existingAuthorSelect, 1);

    v->addLayout(t);

    QHBoxLayout* l = new QHBoxLayout(page, 0, KDialog::spacingHint());

    m_listView = new QListView(page);
    m_listView->setAllColumnsShowFocus(true);
    m_listView->setShowSortIndicator(false);
    m_listView->setDefaultRenameAction( QListView::Accept );
    m_listView->setSortColumn(-1);
    m_listView->addColumn(i18n("Surname" ));
    m_listView->addColumn(i18n("Given names" ));
    m_listView->setColumnWidthMode(0, QListView::Maximum);
    m_listView->setColumnWidthMode(1, QListView::Maximum);
    m_listView->setResizeMode(QListView::AllColumns);
    l->addWidget(m_listView);

    QVBoxLayout* box = new QVBoxLayout(page, 0, KDialog::spacingHint());

    addBtn = new KPushButton("Add", page);
    editBtn = new KPushButton("Edit", page);
    deleteBtn = new KPushButton("Delete", page);
    upBtn = new KPushButton("Up", page);
    downBtn = new KPushButton("Down", page);

    addBtn->setIconSet( QIconSet( SmallIcon( "add" ) ) );
    deleteBtn->setIconSet( QIconSet( SmallIcon( "editdelete" ) ) );
    editBtn->setIconSet( QIconSet( SmallIcon( "edit" ) ) );
    upBtn->setIconSet( QIconSet( SmallIcon( "up" ) ) );
    downBtn->setIconSet( QIconSet( SmallIcon( "down" ) ) );

    QToolTip::add( addBtn, QString( i18n( "Add new author/editor" )  ) );
    QToolTip::add( deleteBtn, QString( i18n( "Delete selected author/editor" )  ) );
    QToolTip::add( editBtn, QString( i18n( "Edit selected author/editor" )  ) );
    QToolTip::add( upBtn, QString( i18n( "Move selected author/editor up" )  ) );
    QToolTip::add( downBtn, QString( i18n( "Move selected author/editor down" )  ) );


    box->addWidget(addBtn);
    box->addWidget(editBtn);
    box->addWidget(deleteBtn);
    box->addWidget(upBtn);
    box->addWidget(downBtn);
    box->addStretch(0);

    l->addLayout(box,0);

    v->addLayout(l);
    //  l->addStretch(0);
    setMainWidget(page);

    setMinimumWidth(400);

    enableButtonSeparator(true);
    updateGUI();

    connect( existingAuthorSelect, SIGNAL( activated ( int ) ), this, SLOT( slotExistingAuthor() ) );
    connect( m_listView, SIGNAL( selectionChanged() ), this, SLOT( updateGUI() ) );
    connect( m_listView, SIGNAL( itemRenamed( QListViewItem*, int, const QString& ) ), this, SLOT( slotItemRenamed( QListViewItem*, int, const QString& ) ) );

    connect(addBtn, SIGNAL(clicked()), this, SLOT(slotAdd()) );
    connect(editBtn, SIGNAL(clicked()), this, SLOT(slotEdit()) );
    connect(deleteBtn, SIGNAL(clicked()), this, SLOT(slotDelete()) );
    connect(upBtn, SIGNAL(clicked()), this, SLOT(slotUp()) );
    connect(downBtn, SIGNAL(clicked()), this, SLOT(slotDown()) );

    //Update author list
    KBibEditEntryDlg *editEntry = (KBibEditEntryDlg*) this->parent();
    KBibApp* win = (KBibApp*) editEntry->parent();
    KBibDoc* doc = win->getDocument();
    QStringList authorList = doc->getAuthorList();
    authorList.sort();
    existingAuthorSelect->insertStringList(authorList);

}


KBibEditAuthors::~KBibEditAuthors()
{
    delete existingAuthorSelect;
    delete m_listView;
    delete addBtn;
    delete editBtn;
    delete deleteBtn;
    delete upBtn;
    delete downBtn;
}

void KBibEditAuthors::setAuthors(QString stx)
{
	QString st = stx.simplifyWhiteSpace();
	QStringList authors = QStringList::split( " and ", st );
    m_listView->clear();
    for ( QStringList::Iterator it = authors.begin(); it != authors.end(); ++it )
    {
        QString author = (*it);
        author = author.stripWhiteSpace();
        QString surname, givenname;
        /*
        if (author.contains(',') ){
        	surname = author.section( ',', 0, 0 );
        	givenname = author.section( ',', 1, 1 );
        }
        else{
        	surname = author.section(' ', -1, -1);
        	givenname = author.section(' ', -100, -2);	
        }

        surname = surname.stripWhiteSpace();
        givenname = givenname.stripWhiteSpace();
        */
        processAuthor(author, surname, givenname);

        QListViewItem* item = new QListViewItem(m_listView, m_listView->lastItem(),  surname, givenname);
        m_listView->ensureItemVisible(item);
        item->setRenameEnabled(0,true);
        item->setRenameEnabled(1,true);

    }
    updateGUI();

}

QString KBibEditAuthors::getAuthors()
{
    QString authors;
    QString surname, givenname;
    bool first = true;
    for(QListViewItem* item = m_listView->firstChild(); item; item = item->nextSibling())
    {
        if (!first)
            authors.append(" and ");
        first = false;
        surname = item->text(0);
        givenname = item->text(1);
        authors.append(surname.simplifyWhiteSpace());
        if (!givenname.isEmpty())
        {
            authors.append(", ");
            authors.append(givenname.simplifyWhiteSpace());
        }

    }
    return authors;

}

void KBibEditAuthors::slotExistingAuthor()
{
    QString author;
    QString surname;
    QString givenname;

    author = existingAuthorSelect->currentText();
    processAuthor(author, surname, givenname);

    QListViewItem* item = new QListViewItem(m_listView, m_listView->lastItem(),  surname, givenname);
    m_listView->ensureItemVisible(item);
    m_listView->setSelected(item, true);
    item->setRenameEnabled(0,true);
    item->setRenameEnabled(1,true);
    updateGUI();
}

void KBibEditAuthors::slotAdd()
{
    QListViewItem* item = new QListViewItem(m_listView, m_listView->lastItem(),  "Surname", "Given names");
    m_listView->ensureItemVisible(item);
    m_listView->setSelected(item, true);
    item->setRenameEnabled(0,true);
    item->setRenameEnabled(1,true);
    updateGUI();
    slotEdit();

}

void KBibEditAuthors::slotEdit()
{
    QListViewItem* item = m_listView->selectedItem();
    if (item)
    {
        item->startRename(0);
        //		item->startRename(1);
        //updateGUI();
    }
}
void KBibEditAuthors::slotDelete()
{
    QListViewItem* item = m_listView->selectedItem();
    if (item)
    {
        m_listView->takeItem(item);
        updateGUI();
    }

}
void KBibEditAuthors::slotUp()
{

    QListViewItem * item = m_listView->selectedItem();
    if ( item != NULL && item -> itemAbove() != NULL )
    {
        item->itemAbove() ->moveItem( item );
        updateGUI();
    }


}
void KBibEditAuthors::slotDown()
{
    QListViewItem* item = m_listView->selectedItem();
    if (item)
    {
        QListViewItem* next_item = item->nextSibling();
        if (next_item)
        {
            item->moveItem(next_item);
        }
        updateGUI();

    }

}

void KBibEditAuthors::updateGUI()
{

    QListViewItem* item = m_listView->selectedItem();
    if (item)
    {
        deleteBtn->setEnabled(true);
        editBtn->setEnabled(true);
        if (item->itemAbove())
            upBtn->setEnabled(true);
        else
            upBtn->setEnabled(false);
        if (item->itemBelow())
            downBtn->setEnabled(true);
        else
            downBtn->setEnabled(false);
    }
    else
    {
        deleteBtn->setEnabled(false);
        editBtn->setEnabled(false);
        upBtn->setEnabled(false);
        downBtn->setEnabled(false);
    }
}

void KBibEditAuthors::slotItemRenamed( QListViewItem * item, int col, const QString & text )
{
    if ( text.isEmpty()  && item != NULL )
    {
        if ( (col == 0 && (item->text(1)).isEmpty() ) || (col == 1 && (item->text(0)).isEmpty() ) )
        {
            delete item;
            updateGUI();
        }
    }
}

#include "kbibeditauthors.moc"

