/***************************************************************************
                          kbibeditentrydlg.cpp  -  description
                             -------------------
    begin                : Sat May 31 2003
    copyright            : (C) 2003 by Thach Nguyen
    email                : thach@dragon.thach.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include "kbibeditentrydlg.h"
#include "kbiblistview.h"
#include "kbib.h"
#include "kbibnewentrydefdlg.h"
#include "kbibdoc.h"
#include "kbibeditauthors.h"
#include "fieldcompletion.h"


extern vector<string> entrydefTab;


KBibEditEntryDlg::KBibEditEntryDlg(BibEntry *entry, QWidget *parent, const char *name)
        :KDialogBase( parent, name , false, i18n("Edit Entry"), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, false )
{

    urlIndex = -1;
    mainTab = new QTabWidget(this);
    setMainWidget(mainTab);
    QWidget *fieldEdit = new  QWidget(mainTab);
    QVBoxLayout* l1 = new QVBoxLayout(fieldEdit, 0, KDialog::spacingHint());
    l1->setMargin(10);
    QHBox* box1 = new QHBox(fieldEdit);
    box1->setMargin(4);
    box1->setSpacing(4);
    mainTab->setMinimumWidth(800);


    entryTypeLabel = new QLabel("Reference Type:", box1);
    entryTypeLabel->setMinimumWidth(140);
    entryTypeSelect = new QComboBox(true, box1, "entryType");
    //editTypeBttn = new QPushButton("Edit Type", box1);
    editTypeBttn = new QPushButton(box1);
    // QPixmap editPix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("edit"), KIcon::Small);
    //editTypeBttn->setPixmap(editPix);
    editTypeBttn->setIconSet( QIconSet( SmallIcon( "edit" ) ) );
    editTypeBttn->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
    QToolTip::add( editTypeBttn, QString( i18n( "Edit current reference type" )  ) );
    QWhatsThis::add( editTypeBttn, QString( i18n( "Edit current reference type" )  ) );
    editTypeBttn->setMinimumWidth(40);
    box1->setStretchFactor(entryTypeLabel, 0);
    box1->setStretchFactor(entryTypeSelect, 1);
    box1->setStretchFactor(editTypeBttn, 0);
    l1->addWidget(box1);
    editTypeBttn->hide();

    QHBox* box2 = new QHBox(fieldEdit);
    box2->setMargin(4);
    box2->setSpacing(4);

    keyLabel = new QLabel("Citation Key:", box2);
    keyLabel->setMinimumWidth(140);
    keyField = new KLineEdit(box2);
    keyField->setMinimumWidth(140);
    createKeyBttn = new QPushButton("Create Key", box2);
    QToolTip::add( createKeyBttn, QString( i18n( "Create citation key" )  ) );
    QWhatsThis::add( createKeyBttn, QString( i18n( "Create citation key" )  ) );
    createKeyBttn->setMinimumWidth(80);
    lockKeyBttn = new QPushButton(box2);
    lockKeyBttn->setMinimumWidth(40);
    box2->setStretchFactor(keyLabel, 0);
    box2->setStretchFactor(keyField, 1);
    box2->setStretchFactor(createKeyBttn, 0);
    box2->setStretchFactor(lockKeyBttn, 0);
    l1->addWidget(box2);

    reqTab = new QTabWidget(fieldEdit);
    l1->addWidget(reqTab);
    reqTab->setMargin(0);
    //  reqTab->setPaletteForegroundColor(QColor(255,0,0));

    rqScrollView = new QScrollView(reqTab);

    rqScrollView->setFrameStyle(0);
    rqScrollView->setResizePolicy(QScrollView::AutoOneFit);
    reqTab->addTab(rqScrollView, "Required Fields");

    rqBox = new QGroupBox(rqScrollView->viewport());
    rqLayout = new QVBoxLayout(rqBox, 0, KDialog::spacingHint());

    rqBox->setFrameStyle(0);
    rqScrollView->addChild(rqBox);

    journalName = new QComboBox(false, rqBox);
    journalName->setPaletteForegroundColor(QColor(0,0,0));
    journalName->hide();

    stringMacroList = new QComboBox(false, this);
    stringMacroList->hide();

    journalSelect = new QPushButton(rqBox);
    //    QPixmap journalPix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("fileopen"), KIcon::Small);
    QToolTip::add( journalSelect, QString( i18n( "Select a journal from the list" )  ) );
    //	journalSelect->setIconSet( QIconSet( SmallIcon( "fileopen" ) ) );
    //    journalSelect->setPixmap(journalPix);

    journalSelect->setIconSet( QIconSet( SmallIcon( "fileopen" ) ) );
    journalSelect->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
    journalSelect->hide();

    createJournalList();
    createStringMacroList();

    optTab = new QTabWidget(fieldEdit);

    l1->addWidget(optTab);

    reqTab->setMargin(0);
    optScrollView = new QScrollView(optTab);
    optScrollView->setFrameStyle(0);
    optScrollView->setResizePolicy(QScrollView::AutoOneFit);
    optTab->addTab(optScrollView, "Optional Fields");

    optBox = new QGroupBox(optScrollView->viewport());
    optLayout = new QVBoxLayout(optBox, 0, KDialog::spacingHint());
    optBox->setFrameStyle(0);

    optScrollView->addChild(optBox);

    urlBrowse = new QPushButton(optBox);
    //QPixmap pix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("bookmark_add"), KIcon::Small);
    //urlBrowse->setPixmap(pix);
    urlBrowse->setIconSet( QIconSet( SmallIcon( "fileopen" ) ) );
    QToolTip::add( urlBrowse, QString( i18n( "Browse for local file " ) ) );
    urlBrowse->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
    urlBrowse->hide();




    connect(editTypeBttn, SIGNAL(clicked()), this, SLOT(slotEditType()) );
    connect(urlBrowse, SIGNAL(clicked()), this, SLOT(slotURLBrowse()) );
    connect(createKeyBttn, SIGNAL(clicked()), this, SLOT(slotCreateKey()) );
    connect(lockKeyBttn, SIGNAL(clicked()), this, SLOT(slotLockKey()) );

    connect(journalSelect, SIGNAL(clicked()), this, SLOT(slotJournalSelect()));

    connect(journalName, SIGNAL(activated(int)), this, SLOT(slotSelectJournalName(int)));
    connect(stringMacroList, SIGNAL(activated(int)), this, SLOT(slotSelectStringMacro(int)));



    rqLabels.setAutoDelete(true);
    optLabels.setAutoDelete(true);
    rqFields.setAutoDelete(true);
    optFields.setAutoDelete(true);
    asStringMacros.setAutoDelete(true);
    stringMacroSelect.setAutoDelete(true);
    editAuthor.setAutoDelete(true);




    rqScrollView->setHScrollBarMode(QScrollView::AlwaysOff);
    optScrollView->setHScrollBarMode(QScrollView::AlwaysOff);

    for (unsigned int i = 0; i<entrydefTab.size(); i++)
    {
        char* st = const_cast<char*>(entrydefTab[i].c_str());

        BibEntryDef *def = getBibEntryDef(st);

        if (!(def->isHidden) && !(def->isSpecial) )
            entryTypeSelect->insertItem (st, -1 );
    }
    entryTypeSelect->setInsertionPolicy(QComboBox::NoInsertion);

    if (entry == 0)
        //m_entry = new BibEntry(const_cast<char*>("article"), "");
        m_entry = new BibEntry(const_cast<char*>(entrydefTab[0].c_str()), "");
    else
        m_entry = new BibEntry(*entry);

    char *entryTypeName =const_cast<char*>(m_entry->getEntryType());

    entryTypeSelect->setCurrentText(entryTypeName);

    mainTab->addTab(fieldEdit, "Main Fields");
    mainTab->setMargin(0);

    //Exttra fields tab
    QWidget* extraFieldTab = new QWidget(mainTab);

    QHBoxLayout* l4 = new QHBoxLayout(extraFieldTab, 0, KDialog::spacingHint());
    l4->setMargin(40);
    extraFieldView = new QListView(extraFieldTab);
    extraFieldView->setAllColumnsShowFocus(true);
    extraFieldView->setShowSortIndicator(false);
    extraFieldView->setDefaultRenameAction( QListView::Accept );
    extraFieldView->setSortColumn(-1);
    extraFieldView->addColumn(i18n("String Macro" ));
    extraFieldView->addColumn(i18n("Name" ));
    extraFieldView->addColumn(i18n("Value" ));
    //extraFieldView->setColumnWidthMode(0, QListView::Manual);
    //extraFieldView->setColumnWidthMode(1, QListView::Manual);
    //extraFieldView->setColumnWidthMode(1, QListView::Maximum);
    //extraFieldView->setResizeMode(QListView::AllColumns);
    l4->addWidget(extraFieldView);

    QVBoxLayout* box = new QVBoxLayout(extraFieldTab, 0, KDialog::spacingHint());

    extraFieldAddBtn = new KPushButton("Add", extraFieldTab);
    extraFieldEditBtn = new KPushButton("Edit", extraFieldTab);
    extraFieldDeleteBtn = new KPushButton("Delete", extraFieldTab);


    extraFieldAddBtn->setIconSet( QIconSet( SmallIcon( "add" ) ) );
    extraFieldDeleteBtn->setIconSet( QIconSet( SmallIcon( "editdelete" ) ) );
    extraFieldEditBtn->setIconSet( QIconSet( SmallIcon( "edit" ) ) );

    box->addWidget(extraFieldAddBtn);
    box->addWidget(extraFieldDeleteBtn);
    box->addWidget(extraFieldEditBtn);

    box->addStretch(0);
    l4->addLayout(box,0);

    connect( extraFieldView, SIGNAL( selectionChanged() ), this, SLOT( updateExtraFieldGUI() ) );
    connect( extraFieldView, SIGNAL( itemRenamed( QListViewItem*, int, const QString& ) ), this, SLOT( slotExtraFielItemRenamed( QListViewItem*, int, const QString& ) ) );

    connect(extraFieldAddBtn, SIGNAL(clicked()), this, SLOT(slotExtraFieldAdd()) );
    connect(extraFieldEditBtn, SIGNAL(clicked()), this, SLOT(slotExtraFieldEdit()) );
    connect(extraFieldDeleteBtn, SIGNAL(clicked()), this, SLOT(slotExtraFieldDelete()) );


    mainTab->addTab(extraFieldTab, "Extra Fields");


    //Abstract Tab
    abstractPage = new QWidget(mainTab);
    QVBoxLayout* l2 = new QVBoxLayout(abstractPage, 10, KDialog::spacingHint());
    abstractEdit = new QTextEdit(abstractPage);
    l2->addWidget(abstractEdit);
    //mainTab->addTab(abstractEdit, "Abstract");
    abstractPage->hide();


    //Note tab
    QWidget* noteTab = new QWidget(mainTab);
    QVBoxLayout* noteLayout = new QVBoxLayout(noteTab, 10, KDialog::spacingHint());
    noteEdit = new QTextEdit(noteTab);
    noteLayout->addWidget(noteEdit);
    mainTab->addTab(noteTab, "Comment");


    //Source tab
    QWidget* page3 = new QWidget(mainTab);
    QVBoxLayout* l3 = new QVBoxLayout(page3, 10, KDialog::spacingHint());
    sourceEdit = new QTextEdit(page3);
    l3->addWidget(sourceEdit);
    mainTab->addTab(page3, "Source");




    macroSelectGrp = new QButtonGroup(this);
    macroSelectGrp->hide();
    editAuthorGrp = new QButtonGroup(this);
    editAuthorGrp->hide();
    connect(macroSelectGrp, SIGNAL(clicked(int)), this, SLOT(slotStringMacroSelect(int)));
    connect(editAuthorGrp, SIGNAL(clicked(int)), this, SLOT(slotEditAuthor(int)));
    connect(entryTypeSelect, SIGNAL(activated(int )), this, SLOT(slotEntryTypeChange(int) ) ) ;
    connect(mainTab,SIGNAL(currentChanged(QWidget *) ), this, SLOT(slotTabChanged() ) );

    prevTabIndex = 0;
    updateDlg(entryTypeName);
    updateContent(m_entry);





    mainTab->setCurrentPage(0);
}

KBibEditEntryDlg::~KBibEditEntryDlg()
{
    prevTabIndex = 0;
    mainTab->setCurrentPage(0);
    if (sourceEdit)
        delete sourceEdit;
    if (abstractEdit)
        delete abstractEdit;


    //delete m_entry;
    if (entryTypeLabel)
        delete entryTypeLabel;
    if (keyLabel)
        delete keyLabel;
    if (keyField)
        delete keyField;
    if (createKeyBttn)
        delete createKeyBttn;
    if (lockKeyBttn)
        delete lockKeyBttn;
    rqFields.clear();

    //    if (rqFields)
    //        delete rqFields;
    optFields.clear();
    //    if (optFields)
    //        delete optFields;

    rqLabels.clear();
    //    delete rqLabels;
    optLabels.clear();
    //    delete optLabels;

    asStringMacros.clear();
    //    delete asStringMacros;
    stringMacroSelect.clear();
    //    delete stringMacroSelect;
    editAuthor.clear();

    editAuthorIndex.clear();

    delete entryTypeSelect;
    delete editTypeBttn;

    delete m_entry;
    delete urlBrowse;
    delete journalSelect;
    delete journalName;

    delete rqLayout;
    delete optLayout;

    delete rqBox;
    delete optBox;


    delete rqScrollView;
    delete optScrollView;

    delete reqTab;
    delete optTab;

    if (mainTab)
        delete mainTab;

}

/** No descriptions */
void KBibEditEntryDlg::updateDlg(char *entryTypeName)
{

    KBibApp* win = (KBibApp*) this->parent();
    KBibDoc* doc = win->getDocument();

    FieldCompletion *authorCompletion=0, *journalCompletion=0, *keywordCompletion=0;

    if ( (win->kbibConfigState).autocompletion)
    {
        QStringList authorList = doc->getAuthorList();
        authorList.sort();
        authorCompletion = new FieldCompletion(true);
        authorCompletion->setDelimiter(QRegExp(QString::fromLatin1("\\s*\\sand\\s\\s*")) );
        authorCompletion->setIgnoreCase(true);
        authorCompletion->setOrder( KCompletion::Sorted );
        authorCompletion->setItems(authorList);

        QStringList journalList = doc->getJournalList();
        journalList.sort();
        journalCompletion = new FieldCompletion(false);
        journalCompletion->setIgnoreCase(true);
        journalCompletion->setOrder( KCompletion::Sorted );
        journalCompletion->setItems(journalList);

        QStringList keywordList = doc->getKeywordList();
        keywordList.sort();
        keywordCompletion = new FieldCompletion(true);
		keywordCompletion->setDelimiter(QRegExp(QString::fromLatin1("\\s*") + (win->kbibConfigState).keywordSeperator + QString::fromLatin1("\\s*")) );
        keywordCompletion->setIgnoreCase(true);
        keywordCompletion->setOrder( KCompletion::Sorted );
        keywordCompletion->setItems(keywordList);
    }

    if (!rqLabels.isEmpty())
        rqLabels.clear();
    if (!optLabels.isEmpty())
        optLabels.clear();
    if (!rqFields.isEmpty())
        rqFields.clear();
    if (!optFields.isEmpty())
        optFields.clear();

    if (!asStringMacros.isEmpty())
        asStringMacros.clear();

    int i = 0;
    //    macroSelectGrp->disconnect();
    while(macroSelectGrp->count() > 0)
    {
        macroSelectGrp->remove(macroSelectGrp->find(i));
        i++;
    }
    i = 0;
    while(editAuthorGrp->count() > 0)
    {
        editAuthorGrp->remove(editAuthorGrp->find(i));
        i++;
    }

    if (!editAuthor.isEmpty())
        editAuthor.clear();
    editAuthorIndex.clear();

    if (!stringMacroSelect.isEmpty())
        stringMacroSelect.clear();

    if (rqLayout)
        delete rqLayout;

    if (optLayout)
        delete optLayout;

    rqLayout = new QVBoxLayout(rqBox, 0, KDialog::spacingHint(), "reqBoxLayout");
    rqLayout->setMargin(10);

    optLayout = new QVBoxLayout(optBox, 0, KDialog::spacingHint(), "optBoxLayout");
    optLayout->setMargin(10);


    abstractPage->hide();
    mainTab->removePage(abstractPage);

    urlBrowse->hide();

    journalSelect->hide();

    if (QString(m_entry->getField("lockkey"))=="Y")
    {
        //lockKeyBttn->setText("Unlock Key");
        //QPixmap pix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("unlock"), KIcon::Small);
        //lockKeyBttn->setPixmap(pix);
        lockKeyBttn->setIconSet( QIconSet( SmallIcon( "unlock" ) ) );
        lockKeyBttn->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
        QToolTip::add( lockKeyBttn, QString( i18n( "Unlock the citation key" )  ) );
        createKeyBttn->setEnabled(false);
        keyField->setEnabled(false);

    }
    else
    {
        //lockKeyBttn->setText("Lock Key");
        //QPixmap pix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("lock"), KIcon::Small);
        //lockKeyBttn->setPixmap(pix);
        lockKeyBttn->setIconSet( QIconSet( SmallIcon( "lock" ) ) );
        lockKeyBttn->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
        QToolTip::add( lockKeyBttn, QString( i18n( "Lock the citation key" )  ) );
        createKeyBttn->setEnabled(true);
        keyField->setEnabled(true);
    }

    BibEntryDef *entryType = getBibEntryDef(entryTypeName);
    int nrq = entryType->nreq;

    if (nrq > 0 )
    {
        reqTab->show();

        for (int i = 0; i< nrq; i++)
        {
            char req[250];
            strcpy(req, entryType->getRequired(i));
            req[0] = toupper(req[0]);
            QHBoxLayout *box = new QHBoxLayout(rqLayout, KDialog::spacingHint(), "reqBoxLayoutRow");

            QLabel *lb = new QLabel(req,  rqBox);
            lb->show();
            rqLabels.append(lb);

            KLineEdit *editField = new KLineEdit(rqBox);
            editField->show();
            rqFields.append(editField);

            QRadioButton *selectBtn = new QRadioButton(i18n("As string macro"), rqBox);
            QToolTip::add( selectBtn, QString( i18n( "Set \"" )  ) + QString(entryType->getRequired(i)) + QString( i18n( "\" as a string macro" ) ));
            selectBtn->show();
            asStringMacros.append(selectBtn);


            box->addWidget(lb);
            box->addWidget(editField);

            if (QString(entryType->getRequired(i)) == "journal")
            {
                journalSelect->show();
                box->addWidget(journalSelect);
                journalSelect->setMinimumWidth(50);
                journalIndex = i;
                if ((win->kbibConfigState).autocompletion)
                {
                    editField->setCompletionObject( journalCompletion );
                    editField->setAutoDeleteCompletionObject( true );
                }
            }

            if (QString(entryType->getRequired(i)) == "author" || QString(entryType->getRequired(i)) == "editor")
            {
                QPushButton *authorEdit = new QPushButton(rqBox);
                QToolTip::add( authorEdit, QString( i18n( "Edit " ) + QString(entryType->getRequired(i)) ) );
                authorEdit->setIconSet( QIconSet( SmallIcon( "edit" ) ) );
                authorEdit->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
                authorEdit->setMinimumWidth(50);
                authorEdit->show();
                editAuthor.append(authorEdit);
                box->addWidget(authorEdit);
                editAuthorGrp->insert(authorEdit);
                editAuthorIndex.append(i);
                if ((win->kbibConfigState).autocompletion)
                {
                    editField->setCompletionObject( authorCompletion );
                    editField->setAutoDeleteCompletionObject( true );
                }
            }


            box->addWidget(selectBtn);

            lb->setMinimumWidth(100);
            box->setStretchFactor(lb, 0);
            box->setStretchFactor(editField, 1);
            box->setStretchFactor(selectBtn, 0);

            if (stringMacroList->count() > 0)
            {
                QPushButton *macroSelect = new QPushButton(i18n("Strings"), rqBox);
                QToolTip::add( macroSelect, QString( i18n( "Select a string macro from the defined list" )  ) );

                macroSelect->show();
                stringMacroSelect.append(macroSelect);
                macroSelectGrp->insert(macroSelect);

                box->addWidget(macroSelect);
                box->setStretchFactor(macroSelect, 0);
            }


        }
        rqLayout->addStretch(1);
    }
    else
        reqTab->hide();

    int nopt = entryType->nopt;
    int numEditField = 0;
    int editorEditIdx = 0;
    for (int i = 0; i< nopt; i++)
    {
        char opt[250];
        strcpy(opt,entryType->getOptional(i));
        opt[0] = toupper(opt[0]);

		if(QString(entryType->getOptional(i)) != "lockkey" && QString(entryType->getOptional(i)) != "comment")
        {
            if (QString(entryType->getOptional(i)) == "abstract")
            {

                abstractPage->show();
                mainTab->insertTab(abstractPage, "Abstract", 2);
                editorEditIdx--;

            }
            else
            {
                QHBoxLayout *box = new QHBoxLayout(optLayout, KDialog::spacingHint(), "optBoxLayoutRow");
                QLabel *lb = new QLabel(opt, optBox);
                lb->show();

                optLabels.append(lb);
                KLineEdit *editField = new KLineEdit(optBox);

                editField->show();

                optFields.append(editField);

                QRadioButton *selectBtn = new QRadioButton(i18n("As string macro"), optBox);
                QToolTip::add( selectBtn, QString( i18n( "Set \"" )  ) + QString(entryType->getOptional(i)) + QString( i18n( "\" as a string macro" ) ));
                selectBtn->show();
                asStringMacros.append(selectBtn);



                box->addWidget(lb);
                box->addWidget(editField);

                if (QString(entryType->getOptional(i)) == "journal")
                {
                    //journalSelect->show();
                    //box->addWidget(journalSelect);
                    journalSelect->setMinimumWidth(50);
                    journalIndex = i+ editorEditIdx;

                    if ((win->kbibConfigState).autocompletion)
                    {
                        editField->setCompletionObject( journalCompletion );
                        editField->setAutoDeleteCompletionObject( true );

                    }
                }
                if (QString(entryType->getOptional(i)) == "author" || QString(entryType->getOptional(i)) == "editor")
                {
                    QPushButton *authorEdit = new QPushButton(optBox);
                    QToolTip::add( authorEdit, QString( i18n( "Edit " ) + QString(entryType->getOptional(i)) ) );
                    authorEdit->setIconSet( QIconSet( SmallIcon( "edit" ) ) );
                    authorEdit->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
                    authorEdit->setMinimumWidth(50);
                    authorEdit->show();
                    editAuthor.append(authorEdit);
                    box->addWidget(authorEdit);
                    editAuthorGrp->insert(authorEdit);
                    editAuthorIndex.append(i+nrq+editorEditIdx);
                    if ((win->kbibConfigState).autocompletion)
                    {
                        editField->setCompletionObject( authorCompletion );
                        editField->setAutoDeleteCompletionObject( true );
                    }
                }

                if (QString(entryType->getOptional(i)) == "keywords")
                {
                    if ((win->kbibConfigState).autocompletion)
                    {
                        editField->setCompletionObject( keywordCompletion );
                        editField->setAutoDeleteCompletionObject( true );
                    }
                }

                if (QString(entryType->getOptional(i)) == "file")
                {

                    urlBrowse->show();
                    box->addWidget(urlBrowse);
                    urlBrowse->setMinimumWidth(50);
                    urlIndex = numEditField;
                }

                box->addWidget(selectBtn);

                lb->setMinimumWidth(100);
                box->setStretchFactor(lb, 0);
                box->setStretchFactor(editField, 1);
                box->setStretchFactor(selectBtn, 0);

                if (stringMacroList->count() > 0)
                {
                    QPushButton *macroSelect = new QPushButton(i18n("Strings"), optBox);
                    QToolTip::add( macroSelect, QString( i18n( "Select a string macro from the defined list" )  ) );

                    macroSelect->show();
                    stringMacroSelect.append(macroSelect);
                    macroSelectGrp->insert(macroSelect);
                    box->addWidget(macroSelect);
                    box->setStretchFactor(macroSelect, 0);
                }

                numEditField++;



            }
        }
        else
            editorEditIdx--;
    }
    optLayout->addStretch(1);
    if (numEditField == 0)
        optTab->hide();
    else
        optTab->show();
}
/** No descriptions */
void KBibEditEntryDlg::updateContent(BibEntry *entry)
{
    QLabel *lb = rqLabels.first();
    KLineEdit *field = rqFields.first();
    QRadioButton *btn = asStringMacros.first();

    keyField->setText(entry->getKey() );

    //do
    while(lb!=0)
    {
        QString st = lb->text();
        field->setText(entry->getField(st) );
        field->setCursorPosition(0);
        btn->setChecked(entry->stringMacroIndicator(st) );
        lb = rqLabels.next();
        field = rqFields.next();
        btn = asStringMacros.next();

    }//while(lb!=0);

    lb = optLabels.first();
    field = optFields.first();
    //do
    while(lb!=0)
    {
        QString st = lb->text();
        field->setText(entry->getField(st) );
        field->setCursorPosition(0);
        btn->setChecked(entry->stringMacroIndicator(st) );
        lb = optLabels.next();
        field = optFields.next();
        btn = asStringMacros.next();

    }//while(lb!=0);

    abstractEdit->setText(entry->getField("abstract") );

    //Extra Field Tab
    extraFieldView->clear();
    for (int i = 0; i < entry->getNoExtraFields(); i++)
    {
        //new QListViewItem( extraFieldView, entry->getFieldName(i+entry->getNoReqFields() + entry->getNoOptFields() ), entry->getExtField( i) );
        QCheckListItem *item = new QCheckListItem( extraFieldView, extraFieldView->lastItem(), "", QCheckListItem::CheckBox);
        char *fn = entry->getFieldName(i+entry->getNoReqFields() + entry->getNoOptFields() );
        item->setText(1, fn );
        item->setText(2, entry->getExtField( i) ) ;
        item->setOn(entry->stringMacroIndicator(fn) );
        extraFieldView->ensureItemVisible(item);
        item->setRenameEnabled(1,true);
        item->setRenameEnabled(2,true);
    }
    updateExtraFieldGUI();

    //Note Tab
	noteEdit->setText(entry->getField("comment") );

}
/** No descriptions */
void KBibEditEntryDlg::slotEntryTypeChange(int index)
{

    QString st = entryTypeSelect->text(index);
    char *type =  const_cast<char*>(st.ascii());

    getBibEntry();
    BibEntry *new_entry = new BibEntry(type, "");
    char *key =  const_cast<char*>(m_entry->getKey());
    new_entry->setKey(key );
    new_entry->setIndex(m_entry->getIndex());
    new_entry->setFields(*m_entry);

    delete m_entry;

    m_entry = new BibEntry(*new_entry);

    updateDlg(type);
    updateContent(m_entry);

}

void KBibEditEntryDlg::accept(void)
{
    if ( ( (mainTab->count() == 3) && (mainTab->currentPageIndex() == 2) ) ||
            ( (mainTab->count() == 2) && (mainTab->currentPageIndex() == 1) ) )
    {
        QString str = sourceEdit->text();
        int dummy;
        BibEntry *tmp_entry = processBibEntryFromString(str.ascii(), dummy );
        if (!tmp_entry)
        {
            KMessageBox::error(this, i18n("Errors in BibTex source.") );
        }
        else
        {
            m_entry = tmp_entry;
            QDialog::accept();
        }

    }
    else
    {

        getBibEntry();
        QDialog::accept();

    }


}

BibEntry * KBibEditEntryDlg::getEditedBibEntry(void)
{
    return m_entry;

}


/** No descriptions */
BibEntry * KBibEditEntryDlg::getBibEntry(void)
{
    QString qst;
    qst = keyField->text();
    qst = qst.stripWhiteSpace();
    char *st = const_cast<char*>(qst.ascii());

    m_entry->setPreamble(false);
    m_entry->setStringMacro(false);
    m_entry->setKey(st);

    KLineEdit *field = rqFields.first();
    QLabel *fieldLabel = rqLabels.first();
    QRadioButton *macro = asStringMacros.first();

    QString lst;

    while(field!=0)
    {
        qst = field->text();
        qst = qst.stripWhiteSpace();
        st = const_cast<char*>(qst.ascii());
        // 		cerr << "\n" << st;
        lst = fieldLabel->text();
        m_entry->setField(const_cast<char*>((lst.lower()).ascii()), st);
        m_entry->setStringMacroIndicator((lst.lower()).ascii(), macro->isChecked() );
        field = rqFields.next();
        fieldLabel=rqLabels.next();
        macro= asStringMacros.next();

    }

    field = optFields.first();
    fieldLabel=optLabels.first();

    while(field!=0)
    {
        qst = field->text();
        qst = qst.stripWhiteSpace();
        st = const_cast<char*>(qst.ascii());
        // 		cerr << "\n" << st;
        lst = fieldLabel->text();
        m_entry->setField(const_cast<char*>((lst.lower()).ascii()), st);
        m_entry->setStringMacroIndicator((lst.lower()).ascii(), macro->isChecked() );
        field = optFields.next();
        fieldLabel=optLabels.next();
        macro= asStringMacros.next();

    }//while(field!=0);

    QListViewItem *it =extraFieldView->firstChild();
    m_entry->clearExtraField();
    for (int i = 0 ; i < extraFieldView->childCount(); i++)
    {

        lst = it->text(1);
        qst = it->text(2);
        qst = qst.stripWhiteSpace();
        st = const_cast<char*>(qst.ascii());

        m_entry->setField(const_cast<char*>((lst.lower()).ascii()), st);
        QCheckListItem *itm = dynamic_cast<QCheckListItem*>(it);
        m_entry->setStringMacroIndicator(const_cast<char*>((lst.lower()).ascii()), itm->isOn());
        it = it->nextSibling();

    }

    //  	cerr << "\n" << (abstractEdit->text()).ascii();
    if ( getBibEntryDef(m_entry->getEntryType())->getFieldIdx("abstract") >= 0)
        m_entry->setField("abstract", const_cast<char*> ( (abstractEdit->text()).ascii() ) );

	m_entry->setField("comment", const_cast<char*> ( (noteEdit->text()).ascii() ) );
    return m_entry;

}

void KBibEditEntryDlg::slotEditType()
{
    char *entryTypeName = const_cast<char*>((entryTypeSelect->currentText()).ascii());

    KbibNewEntryDefDlg * newEntryDefDlg = new KbibNewEntryDefDlg(this);

    newEntryDefDlg->setContent(entryTypeName);

    bool ok = false;
    while(!ok)
    {
        ok = true;
        if (newEntryDefDlg->exec()==QDialog::Accepted)
        {
            char *reqField = newEntryDefDlg->getReqFields();
            char *optField = newEntryDefDlg->getOptFields();
            int n = newEntryDefDlg->getNumReq() + newEntryDefDlg->getNumOpt();
            if(n != 0)
            {
                new_entrydef( entryTypeName, reqField, optField, TRUE);
                writeXMLFile();
                ok = true;
            }
            else
            {
                KMessageBox::error(this, i18n("You did not enter any field!") );
                ok = false;
            }

        }




    }
    delete newEntryDefDlg;


    BibEntry *new_entry = new BibEntry(entryTypeName, "");
    char *key =  const_cast<char*>(m_entry->getKey());
    new_entry->setKey(key );
    new_entry->setFields(*m_entry);

    delete m_entry;

    m_entry = new BibEntry(*new_entry);
    updateDlg(entryTypeName);

    updateContent(m_entry);

}

/** No descriptions */
void KBibEditEntryDlg::slotURLBrowse()
{
    KURL url = KFileDialog::getOpenURL(QString::fromLatin1(":open"), "*|All files",
                                       this, i18n("Open File..."));
    if (!url.isEmpty())
    {
        KLineEdit* urlField  = optFields.at(urlIndex);
        urlField->setText(url.url());
    }
}
/** No descriptions */
void KBibEditEntryDlg::slotCreateKey()
{
    KBibApp* win = (KBibApp*) this->parent();
    getBibEntry();
    if ( (QString(m_entry->getField("author") ).isEmpty() )  && ( QString(m_entry->getField("editor") ).isEmpty() ))
    {
        KMessageBox::error(this, i18n("Author is empty.\nPlease correct it first. ") );
        return;
    }
    m_entry->createKey(((win->kbibConfigState).keyConnecting).ascii());

    //KBibListView *listView = (KBibListView *) parent();
    //QSplitter* vbox = (QSplitter*) listView->parent();


    string fkey;
    char dletter='a';
    int oopscount=0;
    int idx;
    int this_id;

    fkey= m_entry->getKey();
    keyField->setText(fkey);

    idx = win->searchKey(const_cast<char*>(fkey.c_str()));

    this_id = m_entry->getIndex();

    if (idx < 0)
        return;

    if (idx == this_id && idx >=0)
    {
        idx = win->searchKey(const_cast<char*>(fkey.c_str()), this_id+1);
        if (idx < 0)
            return;

    }

    while(1)
    {
        fkey= m_entry->getKey();
        for(int j=0;j<oopscount;j++)
            fkey+='x';
        fkey+=dletter;
        idx = win->searchKey(const_cast<char*>(fkey.c_str()),0);
        if (idx < 0)
        {
            m_entry->setKey(const_cast<char*>(fkey.c_str()));
            keyField->setText(fkey);
            break;
        }
        if (idx == this_id && idx >= 0)
        {
            idx = win->searchKey(const_cast<char*>(fkey.c_str()), this_id+1);
            if (idx < 0)
            {
                m_entry->setKey(const_cast<char*>(fkey.c_str()));
                keyField->setText(fkey);
                break;
            }
        }


        dletter++;
        if (dletter=='z')
        {
            oopscount++;
            dletter='a';
        }

    }
}



void KBibEditEntryDlg::slotLockKey()
{
    if (QString(m_entry->getField("lockkey"))=="Y")
    {
        //lockKeyBttn->setText("Lock Key");
        //QPixmap pix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("lock"), KIcon::Small);
        //lockKeyBttn->setPixmap(pix);
        lockKeyBttn->setIconSet( QIconSet( SmallIcon( "lock" ) ) );
        QToolTip::add( lockKeyBttn, QString( i18n( "Lock the citation key" )  ) );
        createKeyBttn->setEnabled(true);
        keyField->setEnabled(true);
        m_entry->setField("lockkey", "N");
    }
    else
    {
        //lockKeyBttn->setText("Unlock Key");
        //QPixmap pix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("unlock"), KIcon::Small);
        //lockKeyBttn->setPixmap(pix);
        lockKeyBttn->setIconSet( QIconSet( SmallIcon( "unlock" ) ) );
        QToolTip::add( lockKeyBttn, QString( i18n( "Unlock the citation key" )  ) );
        createKeyBttn->setEnabled(false);
        keyField->setEnabled(false);
        m_entry->setField("lockkey", "Y");
    }

}

void KBibEditEntryDlg::slotJournalSelect()
{
    journalName->setGeometry(QRect(journalSelect->x()-240, journalSelect->y(), 300,0));
    //journalName->show();
    journalName->popup();
}

void KBibEditEntryDlg::slotSelectJournalName(int index)
{
    KLineEdit* journalField  = rqFields.at(journalIndex);
    QStringList::Iterator name = journalList.at(index);
    journalField->setText(*name);
    journalName->hide();
    QRadioButton* radioBtn = asStringMacros.at(journalIndex);
    radioBtn->setChecked(TRUE);
}

void KBibEditEntryDlg::slotStringMacroSelect(int idx)
{
    QPushButton *btn = stringMacroSelect.at(idx);

	if (idx < (int) rqFields.count() )
        stringMacroList->setGeometry(QRect(btn->x()-100, btn->y()+reqTab->y() + mainTab->y() + rqScrollView->y()+ rqBox->y()+70, 300,0));
    else
        stringMacroList->setGeometry(QRect(btn->x()-100, btn->y()+optTab->y() + mainTab->y() + optScrollView->y()+ optBox->y()+70, 300,0));
    stringMacroList->popup();
    fieldIdx = idx;
}

void KBibEditEntryDlg::slotSelectStringMacro(int index)
{

    KLineEdit* editField;
	if (fieldIdx < (int) rqFields.count() )
        editField = rqFields.at(fieldIdx);
    else
        editField = optFields.at(fieldIdx - (rqFields.count()));

    KBibApp* win = (KBibApp*) this->parent();
    KBibDoc* m_doc = win->getDocument();
    BibentryTable *entryList = m_doc-> getEntryList();

    BibEntry *entry = entryList->get_stringMacro(index);

    editField->setText(entry->getKey());
    stringMacroList->hide();
    QRadioButton* radioBtn = asStringMacros.at(fieldIdx);
    radioBtn->setChecked(TRUE);

}

void KBibEditEntryDlg::slotTabChanged(void)
{

    if ( mainTab->count() -1 ==  prevTabIndex)
    {

        QString str = sourceEdit->text();
        int dummy;
        BibEntry *tmp_entry = processBibEntryFromString(str.ascii(), dummy );
        if (!tmp_entry)
        {
            KMessageBox::error(this, i18n("Errors in BibTex source.") );
            char *entryTypeName =const_cast<char*>(m_entry->getEntryType());
            updateDlg(entryTypeName );
            updateContent(m_entry);
        }
        else
        {
            tmp_entry->setIndex(m_entry->getIndex());
            delete m_entry;
            m_entry = tmp_entry;
            char *entryTypeName =const_cast<char*>(m_entry->getEntryType());

            if (mainTab->currentPageIndex() == 0 || mainTab->currentPageIndex() == 1 || mainTab->currentPageIndex() == 2)
            {
                entryTypeSelect->setCurrentText(entryTypeName);
                updateDlg(entryTypeName );
                updateContent(m_entry);
            }
            abstractEdit->setText(m_entry->getField("abstract"));
        }

        prevTabIndex = mainTab->currentPageIndex();
    }

    else
    {
        if (mainTab->count() - 1 == mainTab->currentPageIndex())
        {
            getBibEntry();
            if (mainTab->count() == 5 && mainTab->currentPageIndex() == 4)
            {
                abstractPage->hide();
                mainTab->removePage(abstractPage);
            }
            if (m_entry)
            {
                KBibApp* win = (KBibApp*) this->parent();
                QString ld, rd;
                switch ((win->kbibConfigState).fieldDelimiter)
                {
                case 1:
                    rd = ld="\"";
                    break;
                case 2:
                    ld = "(";
                    rd = ")";
                    break;
                default:
                    ld = "{";
                    rd = "}";
                    break;
                }

                QString st = QString();
                st=st+"@"+m_entry->getEntryType()+"{"+m_entry->getKey();
                for (int j=0; j < m_entry-> getNoFields(); j++)
                {
                    if (m_entry->stringMacroIndicator(m_entry->getFieldName(j)) )
                        st=st+",\n\t"+m_entry->getFieldName(j)+"="+m_entry->getField(j);
                    else
                        st=st+",\n\t"+m_entry->getFieldName(j)+"="+ld+m_entry->getField(j)+rd;
                }
                st=st+"\n}\n";
                sourceEdit->setText(st);
            }
            prevTabIndex = mainTab->currentPageIndex();

        }
    }


}

void KBibEditEntryDlg::createJournalList()
{
    journalName->insertItem("--IEEE, Optics, lightwave and photonics--", -1);
    journalName->insertItem("Journal of Lightwave Technology", -1);
    journalName->insertItem("Photonic Technology Letter", -1);

    journalName->insertItem("--IEEE, Electromagnetics, antennas, EMI, magnetics and microwave", -1);
    journalName->insertItem("Antennas Wireless Propagat. Lett.", -1);
    journalName->insertItem("Microwave Guided Wave Lett.", -1);
    journalName->insertItem("Microwave Wireless Compon. Lett.", -1);
    journalName->insertItem("Trans. Antennas Propagat.", -1);
    journalName->insertItem("Trans. Electromagn. Compat.", -1);
    journalName->insertItem("Trans. Magn.", -1);
    journalName->insertItem("Trans. Microwave Theory Tech.", -1);
    journalName->insertItem("Trans. Radio Freq. Interference", -1);
    journalName->insertItem("Transl. J. Magn. Jpn.", -1);

    journalName->insertItem("--IEEE, Physics, electrons, nanotechnology, nuclear and quantum electronics", -1);
    journalName->insertItem("Electron Device Lett.", -1);
    journalName->insertItem("J. Quantum Electron.", -1);
    journalName->insertItem("J. Select. Topics Quantum Electron.", -1);
    journalName->insertItem("Trans. Electron Devices", -1);
    journalName->insertItem("Trans. Nanotechnol.", -1);
    journalName->insertItem("Trans. Nucl. Sci.", -1);
    journalName->insertItem("Trans. Plasma Sci.", -1);
    journalName->insertItem("--IEEE, Communications--", -1);
    journalName->insertItem("Commun. Lett.", -1);
    journalName->insertItem("J. Select. Areas Commun.", -1);
    journalName->insertItem("Trans. Commun.", -1);
    journalName->insertItem("Trans. Commun. Technol.", -1);
    journalName->insertItem("Trans. Wireless Commun.", -1);


    journalList.append("");
    journalList.append("IEEE_J_JLT");
    journalList.append("IEEE_J_PTL");
    journalList.append("");
    journalList.append("IEEE_J_AWPL");
    journalList.append("IEEE_J_MGWL");
    journalList.append("IEEE_J_MWCL");
    journalList.append("IEEE_J_AP");
    journalList.append("IEEE_J_EMC");
    journalList.append("IEEE_J_MAG");
    journalList.append("IEEE_J_MTT");
    journalList.append("IEEE_J_RFI");
    journalList.append("IEEE_J_TJMJ");

    journalList.append("");
    journalList.append("IEEE_J_EDL");
    journalList.append("IEEE_J_JQE");
    journalList.append("IEEE_J_JSTQE");
    journalList.append("IEEE_J_ED");
    journalList.append("IEEE_J_NANO");
    journalList.append("IEEE_J_NS");
    journalList.append("IEEE_J_PS");
    journalList.append("");
    journalList.append("IEEE_J_COML");
    journalList.append("IEEE_J_JSAC");
    journalList.append("IEEE_J_COM");
    journalList.append("IEEE_J_COMT");
    journalList.append("IEEE_J_WCOM");

}

void KBibEditEntryDlg::createStringMacroList()
{
    KBibApp* win = (KBibApp*) this->parent();
    KBibDoc* m_doc = win->getDocument();
    BibentryTable *entryList = m_doc-> getEntryList();

    int size = entryList->stringMacroSize();
    BibEntry *entry;
    for(int i = 0; i < size; i++)
    {
        entry = entryList->get_stringMacro(i);
        stringMacroList->insertItem(entry->getField(0) );
    }
}


void KBibEditEntryDlg::slotEditAuthor(int i)
{
    int index = *(editAuthorIndex.at(i));
    KLineEdit* editField;
	if (index < (int) rqFields.count() )
        editField = rqFields.at(index);
    else
        editField = optFields.at(index - (rqFields.count()));


    KBibEditAuthors * editAuthorsDlg = new KBibEditAuthors(this);
    editAuthorsDlg->setAuthors(editField->text());

    if (editAuthorsDlg->exec()==QDialog::Accepted)
    {
        editField->setText(editAuthorsDlg->getAuthors() );
    }
    delete editAuthorsDlg;

}

void KBibEditEntryDlg::slotExtraFieldAdd()
{
    QCheckListItem *item = new QCheckListItem( extraFieldView, extraFieldView->lastItem(),  "", QCheckListItem::CheckBox);
    item->setText(1, "FieldName" );
    item->setText(2, "Field Value" ) ;
    extraFieldView->ensureItemVisible(item);
    extraFieldView->setSelected(item, true);
    item->setRenameEnabled(1,true);
    item->setRenameEnabled(2,true);
    updateExtraFieldGUI();
    slotExtraFieldEdit();

}
void KBibEditEntryDlg::slotExtraFieldEdit()
{
    QListViewItem* item = extraFieldView->selectedItem();
    if (item)
    {
        item->startRename(1);
        //		item->startRename(1);
        //updateGUI();
    }
}
void KBibEditEntryDlg::slotExtraFieldDelete()
{
    QListViewItem* item = extraFieldView->selectedItem();
    if (item)
    {
        extraFieldView->takeItem(item);
        updateExtraFieldGUI();
    }
}


void KBibEditEntryDlg::slotExtraFielItemRenamed( QListViewItem * item, int col, const QString & text )
{
    if ( text.isEmpty()  && item != NULL )
    {
        if ( (col == 1 && (item->text(2)).isEmpty() ) || (col == 2 && (item->text(1)).isEmpty() ) )
        {
            delete item;
            updateExtraFieldGUI();
        }
    }
}

void KBibEditEntryDlg::updateExtraFieldGUI()
{
    QListViewItem* item = extraFieldView->selectedItem();
    if (item)
    {
        extraFieldDeleteBtn->setEnabled(true);
        extraFieldEditBtn->setEnabled(true);

    }
    else
    {
        extraFieldDeleteBtn->setEnabled(false);
        extraFieldEditBtn->setEnabled(false);
    }
}
