/***************************************************************************
                          kbibinsertfiledlg  -  description
                             -------------------
    begin                : Tue Apr 13 2004
    copyright            : (C) 2004 by Thach Nguyen
    email                : thach@dragon.thach.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "kbibinsertfiledlg.h"
#include <kiconloader.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <qlayout.h>
#include <qhbox.h>

KbibInsertFileDlg::KbibInsertFileDlg(QWidget *parent, const char *name)
    :KDialogBase( parent, name , true, i18n("Insert File"), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, false )
{
  QWidget* page = new QWidget(this);
  setMainWidget(page);
  QVBoxLayout* l = new QVBoxLayout(page, 0, KDialog::spacingHint());

  fileNameLabel = new QLabel("File name:", page);
  l->addWidget(fileNameLabel);

  QHBox* box1 = new QHBox(page);
  box1->setMargin(4);
  box1->setSpacing(4);



  fileName = new KLineEdit(0, box1);
  browseBttn = new KPushButton("...", box1) ;
  QPixmap pix = KGlobal::iconLoader()->loadIcon(QString::fromLatin1("fileopen"), KIcon::Small);
  browseBttn->setPixmap(pix);

  browseBttn->setMinimumWidth(40);
  box1->setStretchFactor(fileName, 1);
  box1->setStretchFactor(browseBttn, 0);

  l->addWidget(box1);

  QHBox* box2 = new QHBox(page);
  box2->setMargin(4);
  box2->setSpacing(4);

  fileTypeLabel=  new QLabel("File format:", box2);


  fileType = new KComboBox(false, box2);
  fileType->insertItem("BibTex", -1);
  fileType->insertItem("Endnote exported BibTex", -1);
  fileType->insertItem("Endnote/Refer", -1);
  fileType->insertItem("RIS", -1);
  fileType->insertItem("ISI", -1);
  fileType->insertItem("Pubmed XML", -1);
  fileType->insertItem("COPAC", -1);
  fileType->insertItem("MODS XML", -1);
  
  fileTypeLabel->setMinimumWidth(80);
  box2->setStretchFactor(fileTypeLabel, 0);
  box2->setStretchFactor(fileType, 1);


  l->addWidget(box2);



  noImportExisting = new QCheckBox(i18n("Do not import existing references"), page);
  noImportExisting->setChecked(TRUE);
  l->addWidget(noImportExisting);
  l->addSpacing(10);
  l->addStretch(1);


  connect(browseBttn, SIGNAL(clicked()), this, SLOT(slotBrowseBttn()) );


}


KbibInsertFileDlg::~KbibInsertFileDlg()
{
  delete fileName;
  delete browseBttn;
  delete fileNameLabel;
  delete fileTypeLabel;
  delete fileType;
  delete noImportExisting;
}

QString KbibInsertFileDlg::getFileName()
{
  return (fileName->text());

}

int KbibInsertFileDlg::getFileType()
{
  return fileType->currentItem();
}

bool KbibInsertFileDlg::getNotInsertExisting()
{
  return noImportExisting->isChecked();
}

#include "kbibinsertfiledlg.moc"


/*!
    \fn KbibInsertFileDlg::slotBrowseBttn()
 */
void KbibInsertFileDlg::slotBrowseBttn()
{
  QString filter = i18n("*.bib|BibTex files (*.bib)");
  filter += QString::fromLatin1("\n");
  filter += i18n("*.end|Endnote files (*.end)");
  filter += QString::fromLatin1("\n");
  filter += i18n("*.ris|RIS files (*.ris)");
  filter += QString::fromLatin1("\n");
  filter += i18n("*.isi|ISI files (*.isi)");
  filter += QString::fromLatin1("\n");
  filter += i18n("*.xml|XML files (*.xml)");
  filter += QString::fromLatin1("\n");
  filter += i18n("*.txt|Text files (*.txt)");
  filter += QString::fromLatin1("\n");
  filter += i18n("*|All files");


  KURL url = KFileDialog::getOpenURL(QString::fromLatin1(":open"), filter,
                                     this, i18n("Open File..."));
  QString fn = url.url();
  if (!url.isEmpty())
  {
    fileName->setText(fn);
  }

  if (fn.contains(".bib"))
    fileType->setCurrentItem(0);
  else if (fn.contains(".end"))
    fileType->setCurrentItem(2);
  else if (fn.contains(".ris"))
    fileType->setCurrentItem(3);
  else if (fn.contains(".isi"))
    fileType->setCurrentItem(4);
  else if (fn.contains(".xml"))
	  fileType->setCurrentItem(7); 
}
