//
// C++ Implementation: newentryfromtextdialog
//
// Description: 
//
//
// Author: Thach Nguyen <thach.nguyen@rmit.edu.au>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "newentryfromtextdialog.h"
#include "bibentrytable.h"
#include "bibentry.h"
#include "entrydef.h"

#include <kcombobox.h>
#include <kpushbutton.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kurl.h>
#include <ktempfile.h>
#include <kmessagebox.h>
#include <kio/netaccess.h>
#include <kio/job.h>
#include <kaccelmanager.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qwhatsthis.h>
#include <qtextedit.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qfile.h>
#include <qtextstream.h>


extern vector<string> entrydefTab;

NewEntryFromTextDialog::NewEntryFromTextDialog(QWidget *parent, const char *name)
	: KDialogBase( parent, name , true, i18n("New Reference from Plain Text"), KDialogBase::Ok|KDialogBase::Cancel, KDialogBase::Ok, false )
{
	QWidget* page = new QWidget(this);
	setMainWidget(page);
	QVBoxLayout* l = new QVBoxLayout(page, 0, KDialog::spacingHint());
	
	QHBox* box1 = new QHBox(page);
	box1->setSpacing(KDialog::spacingHint());
	
	m_clear = new KPushButton(KStdGuiItem::clear(), box1);
	connect(m_clear, SIGNAL(clicked()), SLOT(slotClearClicked()));
	QWhatsThis::add(m_clear, i18n("Click to clear the content of the below text box"));

	
	//m_open = new KPushButton(i18n("Open"), box1);
	//m_open->setGuiItem(KGuiItem(i18n("Open"), SmallIconSet(QString::fromLatin1("fileopen"))));
	m_open = new KPushButton(KStdGuiItem::open(), box1);
	connect(m_open, SIGNAL(clicked()), SLOT(slotOpenClicked()));
	QWhatsThis::add(m_open, i18n("Click to set the content of an existing file into the below text box"));

	m_paste = new KPushButton(i18n("Paste"), box1);
	m_paste->setGuiItem(KGuiItem(i18n("Paste"),
					   SmallIconSet(QString::fromLatin1("editpaste"))));
	connect(m_paste, SIGNAL(clicked()), SLOT(slotPasteClicked()));
	QWhatsThis::add(m_paste, i18n("Click to paste the clipboard content into the below text box"));
	box1->setStretchFactor(new QWidget(box1),1);
	
	l->addWidget(box1);
	
	QHBox* box2 = new QHBox(page);
	box2->setSpacing(KDialog::spacingHint());
	m_edit = new QTextEdit(box2);
	m_edit->setMinimumWidth(400);
	m_edit->setMinimumHeight(400);
//	connect(m_edit, SIGNAL(selectionChanged()), SLOT(slotSelectionChanged()));
	QVBox* v = new QVBox(box2);
	v->setMargin(KDialog::marginHint());
	v->setSpacing(KDialog::spacingHint());
	
	m_entryType = new KComboBox(false, v);
	QWhatsThis::add(m_entryType, i18n("Select the reference type"));
	m_assign = new KPushButton(i18n("Set Selected Text To"), v);
	QWhatsThis::add(m_assign, i18n("Set the selected text into field"));
	connect(m_assign, SIGNAL(clicked()), SLOT(slotSetFieldClicked()));
//	m_assign->setEnabled(false);
	
	v->setStretchFactor(new QWidget(v),1);
	l->addWidget(box2);
	
	for (unsigned int i = 0; i<entrydefTab.size(); i++)
	{
		char* st = const_cast<char*>(entrydefTab[i].c_str());

		BibEntryDef *def = getBibEntryDef(st);

		if (!(def->isHidden) && !(def->isSpecial) )
			m_entryType->insertItem (QString::fromLatin1(st), -1 );
	}
	connect(m_entryType, SIGNAL(activated(int )), SLOT(slotEntryTypeChange(int)));
	
	
	m_fieldMenu = new KPopupMenu(this);
	m_fieldMenu->setCheckable(true);
	m_fieldMenu->insertTitle(i18n("Set selected text to"));
	m_assign->setPopup(m_fieldMenu);
	connect(m_fieldMenu, SIGNAL(activated(int)),
			this, SLOT(slotFieldMenuActivated(int)));
	
	m_edit->setFocus();
	KAcceleratorManager::manage(page);
	
	m_entryType->setCurrentItem(0);
	m_entry = new BibEntry( (m_entryType->text(0) ).ascii(), "");
	slotEntryTypeChange(0);
}


NewEntryFromTextDialog::~NewEntryFromTextDialog()
{
	if (m_entry)
		delete m_entry;
}

void NewEntryFromTextDialog::slotClearClicked()
{
	m_edit->clear();	
}
void NewEntryFromTextDialog::slotOpenClicked()
{
	KURL url = KFileDialog::getOpenURL(QString::fromLatin1(":open"), i18n("*.*|All files"),
									   this, i18n("Open File..."));
	if (!url.isEmpty())
	{
		KTempFile tmpfile;
		tmpfile.setAutoDelete(TRUE);
		QString fn = tmpfile.name();
		if(!KIO::NetAccess::download(url, fn, this))
		{
			QString str;
			if(url.isLocalFile())
			{
				str = i18n("Kbib is unable to find the file - %1.").arg(url.fileName());
			}
			else
			{
				str = i18n("Kbib is unable to download the file - %1.").arg(url.url());
			}
			KMessageBox::sorry(this, str);

			return;
			
		}
		QFile file(fn); // Read the text from a file
		if ( file.open( IO_ReadOnly ) ) {
			QTextStream stream( &file );
			m_edit->setText( stream.read() );
		}
	}
	

}

void NewEntryFromTextDialog::slotPasteClicked()
{
	m_edit->paste();
}

void NewEntryFromTextDialog::slotSelectionChanged()
{
	if (m_edit->hasSelectedText() ){
		m_assign->setEnabled(true);
	}
	else
		m_assign->setEnabled(false);
}

void NewEntryFromTextDialog::slotEntryTypeChange(int index){
	m_fieldMenu->clear();
	QString st = m_entryType->text(index);
	char *type =  const_cast<char*>(st.ascii());
	
	BibEntry *new_entry = new BibEntry(type, "");
	char *key =  const_cast<char*>(m_entry->getKey());
	new_entry->setKey(key );
	new_entry->setIndex(m_entry->getIndex());
	new_entry->setFields(*m_entry);

	delete m_entry;

	m_entry = new BibEntry(*new_entry);
		
	BibEntryDef *entryType = getBibEntryDef(type);
	int nrq = entryType->nreq;
	m_fieldMenu->insertItem(QString::fromLatin1("Citation key"), 0);
	if (!QString(m_entry->getKey()).isEmpty() )
			m_fieldMenu->setItemChecked(0, false);	
	for (int i = 0; i < nrq; i++){
		if (QString::fromLatin1(entryType->getRequired(i)) != QString::fromLatin1("lockkey")){
			m_fieldMenu->insertItem(entryType->getRequired(i), i+1);	
			if (!QString(m_entry->getField(entryType->getRequired(i))).isEmpty())
				m_fieldMenu->setItemChecked(i+1, true);	
		}
	}
	int nopt = entryType->nopt;
	for (int i = 0; i < nopt; i++){
		if (QString::fromLatin1(entryType->getOptional(i)) != QString::fromLatin1("lockkey")){
			m_fieldMenu->insertItem(entryType->getOptional(i), i+nrq+1);	
			if (!QString(m_entry->getField(entryType->getOptional(i))).isEmpty())
				m_fieldMenu->setItemChecked(i+nrq+1, true);
		}
	}

}

void NewEntryFromTextDialog::slotSetFieldClicked()
{
	m_fieldMenu->popup(QWidget::mapToParent (m_assign->pos()) );		
}

void NewEntryFromTextDialog::slotFieldMenuActivated(int idx){
	QString text = (m_edit->selectedText() ).stripWhiteSpace();
	if (text.isEmpty())
		return;
	QString field = m_fieldMenu->text(idx);
	
	if (field == QString::fromLatin1("Citation key") ){
		m_entry->setKey(const_cast<char*>(text.ascii()) );	
	}
	else{
		m_entry->setField(const_cast<char*>(field.ascii()), const_cast<char*>(text.ascii()) );	
	}
	m_fieldMenu->setItemChecked(idx, true);
	m_edit->removeSelection();
//	slotSelectionChanged();
}

#include "newentryfromtextdialog.moc"
